@@ -170,13 +170,6 @@ var ctests = []struct {
 			cfg.AddAptSource("keyName", "someKey")
 		},
 	},
-	{
-		"AptSources",
-		"apt_sources:\n- source: keyName\n  keyid: someKey\n  keyserver: foo.com\n",
-		func(cfg *cloudinit.Config) {
-			cfg.AddAptSourceWithKeyId("keyName", "someKey", "foo.com")
-		},
-	},
 	{
 		"Packages",
 		"packages:\n- juju\n- ubuntu\n",
