@@ -1,18 +1,22 @@
+// Copyright 2012, 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package main
 
 import (
 	"errors"
 	"fmt"
+
 	"launchpad.net/juju-core/cmd"
 	"launchpad.net/juju-core/juju"
-	"launchpad.net/juju-core/state"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/statecmd"
 )
 
 // DestroyUnitCommand is responsible for destroying service units.
 type DestroyUnitCommand struct {
-	EnvCommandBase
+	cmd.EnvCommandBase
 	UnitNames []string
 }
 
@@ -31,7 +35,7 @@ func (c *DestroyUnitCommand) Init(args []string) error {
 		return errors.New("no units specified")
 	}
 	for _, name := range c.UnitNames {
-		if !state.IsUnitName(name) {
+		if !names.IsUnit(name) {
 			return fmt.Errorf("invalid unit name %q", name)
 		}
 	}
@@ -40,7 +44,7 @@ func (c *DestroyUnitCommand) Init(args []string) error {
 
 // Run connects to the environment specified on the command line and destroys
 // units therein.
-func (c *DestroyUnitCommand) Run(_ *cmd.Context) (err error) {
+func (c *DestroyUnitCommand) Run(_ *cmd.Context) error {
 	conn, err := juju.NewConnFromName(c.EnvName)
 	if err != nil {
 		return err
