@@ -4,6 +4,7 @@
 package maas
 
 import (
+	"bytes"
 	"encoding/base64"
 	"fmt"
 	"io/ioutil"
@@ -26,7 +27,6 @@ import (
 	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/juju/testing"
 	"launchpad.net/juju-core/provider/common"
-	coretesting "launchpad.net/juju-core/testing"
 	jc "launchpad.net/juju-core/testing/checkers"
 	"launchpad.net/juju-core/tools"
 	"launchpad.net/juju-core/utils"
@@ -54,31 +54,12 @@ func getTestConfig(name, server, oauth, secret string) *config.Config {
 	return ecfg.Config
 }
 
-// makeEnviron creates a functional maasEnviron for a test.
-func (suite *environSuite) makeEnviron() *maasEnviron {
-	attrs := coretesting.FakeConfig().Merge(coretesting.Attrs{
-		"name":        suite.environ.Name(),
-		"type":        "maas",
-		"maas-oauth":  "a:b:c",
-		"maas-server": suite.testMAASObject.TestServer.URL,
-	})
-	cfg, err := config.New(config.NoDefaults, attrs)
-	if err != nil {
-		panic(err)
-	}
-	env, err := NewEnviron(cfg)
-	if err != nil {
-		panic(err)
-	}
-	return env
-}
-
 func (suite *environSuite) setupFakeProviderStateFile(c *gc.C) {
 	suite.testMAASObject.TestServer.NewFile(common.StateFile, []byte("test file content"))
 }
 
 func (suite *environSuite) setupFakeTools(c *gc.C) {
-	stor := NewStorage(suite.environ)
+	stor := NewStorage(suite.makeEnviron())
 	envtesting.UploadFakeTools(c, stor)
 }
 
@@ -139,7 +120,7 @@ func (suite *environSuite) TestInstancesReturnsInstances(c *gc.C) {
 	resourceURI, _ := node.GetField("resource_uri")
 	instanceIds := []instance.Id{instance.Id(resourceURI)}
 
-	instances, err := suite.environ.Instances(instanceIds)
+	instances, err := suite.makeEnviron().Instances(instanceIds)
 
 	c.Check(err, gc.IsNil)
 	c.Check(len(instances), gc.Equals, 1)
@@ -149,7 +130,7 @@ func (suite *environSuite) TestInstancesReturnsInstances(c *gc.C) {
 func (suite *environSuite) TestInstancesReturnsErrNoInstancesIfEmptyParameter(c *gc.C) {
 	input := `{"system_id": "test"}`
 	suite.testMAASObject.TestServer.NewNode(input)
-	instances, err := suite.environ.Instances([]instance.Id{})
+	instances, err := suite.makeEnviron().Instances([]instance.Id{})
 
 	c.Check(err, gc.Equals, environs.ErrNoInstances)
 	c.Check(instances, gc.IsNil)
@@ -158,14 +139,14 @@ func (suite *environSuite) TestInstancesReturnsErrNoInstancesIfEmptyParameter(c
 func (suite *environSuite) TestInstancesReturnsErrNoInstancesIfNilParameter(c *gc.C) {
 	input := `{"system_id": "test"}`
 	suite.testMAASObject.TestServer.NewNode(input)
-	instances, err := suite.environ.Instances(nil)
+	instances, err := suite.makeEnviron().Instances(nil)
 
 	c.Check(err, gc.Equals, environs.ErrNoInstances)
 	c.Check(instances, gc.IsNil)
 }
 
 func (suite *environSuite) TestInstancesReturnsErrNoInstancesIfNoneFound(c *gc.C) {
-	_, err := suite.environ.Instances([]instance.Id{"unknown"})
+	_, err := suite.makeEnviron().Instances([]instance.Id{"unknown"})
 	c.Check(err, gc.Equals, environs.ErrNoInstances)
 }
 
@@ -174,7 +155,7 @@ func (suite *environSuite) TestAllInstancesReturnsAllInstances(c *gc.C) {
 	node := suite.testMAASObject.TestServer.NewNode(input)
 	resourceURI, _ := node.GetField("resource_uri")
 
-	instances, err := suite.environ.AllInstances()
+	instances, err := suite.makeEnviron().AllInstances()
 
 	c.Check(err, gc.IsNil)
 	c.Check(len(instances), gc.Equals, 1)
@@ -182,7 +163,7 @@ func (suite *environSuite) TestAllInstancesReturnsAllInstances(c *gc.C) {
 }
 
 func (suite *environSuite) TestAllInstancesReturnsEmptySliceIfNoInstance(c *gc.C) {
-	instances, err := suite.environ.AllInstances()
+	instances, err := suite.makeEnviron().AllInstances()
 
 	c.Check(err, gc.IsNil)
 	c.Check(len(instances), gc.Equals, 0)
@@ -198,7 +179,7 @@ func (suite *environSuite) TestInstancesReturnsErrorIfPartialInstances(c *gc.C)
 	instanceId2 := instance.Id("unknown systemID")
 	instanceIds := []instance.Id{instanceId1, instanceId2}
 
-	instances, err := suite.environ.Instances(instanceIds)
+	instances, err := suite.makeEnviron().Instances(instanceIds)
 
 	c.Check(err, gc.Equals, environs.ErrPartialInstances)
 	c.Check(len(instances), gc.Equals, 1)
@@ -298,7 +279,7 @@ func stringp(val string) *string {
 }
 
 func (suite *environSuite) TestAcquireNode(c *gc.C) {
-	stor := NewStorage(suite.environ)
+	stor := NewStorage(suite.makeEnviron())
 	fakeTools := envtesting.MustUploadFakeToolsVersions(stor, version.Current)[0]
 	env := suite.makeEnviron()
 	suite.testMAASObject.TestServer.NewNode(`{"system_id": "node0", "hostname": "host0"}`)
@@ -313,7 +294,7 @@ func (suite *environSuite) TestAcquireNode(c *gc.C) {
 }
 
 func (suite *environSuite) TestAcquireNodeTakesConstraintsIntoAccount(c *gc.C) {
-	stor := NewStorage(suite.environ)
+	stor := NewStorage(suite.makeEnviron())
 	fakeTools := envtesting.MustUploadFakeToolsVersions(stor, version.Current)[0]
 	env := suite.makeEnviron()
 	suite.testMAASObject.TestServer.NewNode(`{"system_id": "node0", "hostname": "host0"}`)
@@ -329,6 +310,21 @@ func (suite *environSuite) TestAcquireNodeTakesConstraintsIntoAccount(c *gc.C) {
 	c.Assert(nodeRequestValues[0].Get("mem"), gc.Equals, "1024")
 }
 
+func (suite *environSuite) TestAcquireNodePassedAgentName(c *gc.C) {
+	stor := NewStorage(suite.makeEnviron())
+	fakeTools := envtesting.MustUploadFakeToolsVersions(stor, version.Current)[0]
+	env := suite.makeEnviron()
+	suite.testMAASObject.TestServer.NewNode(`{"system_id": "node0", "hostname": "host0"}`)
+
+	_, _, err := env.acquireNode(constraints.Value{}, tools.List{fakeTools})
+
+	c.Check(err, gc.IsNil)
+	requestValues := suite.testMAASObject.TestServer.NodeOperationRequestValues()
+	nodeRequestValues, found := requestValues["node0"]
+	c.Assert(found, gc.Equals, true)
+	c.Assert(nodeRequestValues[0].Get("agent_name"), gc.Equals, exampleAgentName)
+}
+
 func (*environSuite) TestConvertConstraints(c *gc.C) {
 	var testValues = []struct {
 		constraints    constraints.Value
@@ -352,13 +348,13 @@ func (*environSuite) TestConvertConstraints(c *gc.C) {
 func (suite *environSuite) getInstance(systemId string) *maasInstance {
 	input := `{"system_id": "` + systemId + `"}`
 	node := suite.testMAASObject.TestServer.NewNode(input)
-	return &maasInstance{&node, suite.environ}
+	return &maasInstance{&node, suite.makeEnviron()}
 }
 
 func (suite *environSuite) TestStopInstancesReturnsIfParameterEmpty(c *gc.C) {
 	suite.getInstance("test1")
 
-	err := suite.environ.StopInstances([]instance.Instance{})
+	err := suite.makeEnviron().StopInstances([]instance.Instance{})
 	c.Check(err, gc.IsNil)
 	operations := suite.testMAASObject.TestServer.NodeOperations()
 	c.Check(operations, gc.DeepEquals, map[string][]string{})
@@ -370,7 +366,7 @@ func (suite *environSuite) TestStopInstancesStopsAndReleasesInstances(c *gc.C) {
 	suite.getInstance("test3")
 	instances := []instance.Instance{instance1, instance2}
 
-	err := suite.environ.StopInstances(instances)
+	err := suite.makeEnviron().StopInstances(instances)
 
 	c.Check(err, gc.IsNil)
 	operations := suite.testMAASObject.TestServer.NodeOperations()
@@ -383,7 +379,7 @@ func (suite *environSuite) TestStateInfo(c *gc.C) {
 	hostname := "test"
 	input := `{"system_id": "system_id", "hostname": "` + hostname + `"}`
 	node := suite.testMAASObject.TestServer.NewNode(input)
-	testInstance := &maasInstance{&node, suite.environ}
+	testInstance := &maasInstance{&node, suite.makeEnviron()}
 	err := common.SaveState(
 		env.Storage(),
 		&common.BootstrapState{StateInstances: []instance.Id{testInstance.Id()}})
@@ -480,7 +476,9 @@ func (suite *environSuite) TestGetImageMetadataSources(c *gc.C) {
 	// Add a dummy file to storage so we can use that to check the
 	// obtained source later.
 	data := makeRandomBytes(10)
-	suite.testMAASObject.TestServer.NewFile("filename", data)
+	stor := NewStorage(env)
+	err := stor.Put("filename", bytes.NewBuffer([]byte(data)), int64(len(data)))
+	c.Assert(err, gc.IsNil)
 	sources, err := imagemetadata.GetMetadataSources(env)
 	c.Assert(err, gc.IsNil)
 	c.Assert(len(sources), gc.Equals, 2)
@@ -495,7 +493,9 @@ func (suite *environSuite) TestGetToolsMetadataSources(c *gc.C) {
 	// Add a dummy file to storage so we can use that to check the
 	// obtained source later.
 	data := makeRandomBytes(10)
-	suite.testMAASObject.TestServer.NewFile("tools/filename", data)
+	stor := NewStorage(env)
+	err := stor.Put("tools/filename", bytes.NewBuffer([]byte(data)), int64(len(data)))
+	c.Assert(err, gc.IsNil)
 	sources, err := envtools.GetMetadataSources(env)
 	c.Assert(err, gc.IsNil)
 	c.Assert(len(sources), gc.Equals, 1)
