@@ -9,7 +9,9 @@ import (
 	gc "launchpad.net/gocheck"
 	"launchpad.net/gomaasapi"
 
+	"launchpad.net/juju-core/environs/config"
 	envtesting "launchpad.net/juju-core/environs/testing"
+	coretesting "launchpad.net/juju-core/testing"
 	"launchpad.net/juju-core/testing/testbase"
 )
 
@@ -20,7 +22,6 @@ func TestMAAS(t *stdtesting.T) {
 type providerSuite struct {
 	testbase.LoggingSuite
 	envtesting.ToolsFixture
-	environ         *maasEnviron
 	testMAASObject  *gomaasapi.TestMAASObject
 	restoreTimeouts func()
 }
@@ -32,7 +33,6 @@ func (s *providerSuite) SetUpSuite(c *gc.C) {
 	s.LoggingSuite.SetUpSuite(c)
 	TestMAASObject := gomaasapi.NewTestMAAS("1.0")
 	s.testMAASObject = TestMAASObject
-	s.environ = &maasEnviron{name: "test env", maasClientUnlocked: &TestMAASObject.MAASObject}
 }
 
 func (s *providerSuite) SetUpTest(c *gc.C) {
@@ -51,3 +51,25 @@ func (s *providerSuite) TearDownSuite(c *gc.C) {
 	s.restoreTimeouts()
 	s.LoggingSuite.TearDownSuite(c)
 }
+
+const exampleAgentName = "dfb69555-0bc4-4d1f-85f2-4ee390974984"
+
+// makeEnviron creates a functional maasEnviron for a test.
+func (suite *providerSuite) makeEnviron() *maasEnviron {
+	attrs := coretesting.FakeConfig().Merge(coretesting.Attrs{
+		"name":            "test env",
+		"type":            "maas",
+		"maas-oauth":      "a:b:c",
+		"maas-server":     suite.testMAASObject.TestServer.URL,
+		"maas-agent-name": exampleAgentName,
+	})
+	cfg, err := config.New(config.NoDefaults, attrs)
+	if err != nil {
+		panic(err)
+	}
+	env, err := NewEnviron(cfg)
+	if err != nil {
+		panic(err)
+	}
+	return env
+}
