@@ -1,7 +1,12 @@
+// Copyright 2012, 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package state
 
 import (
 	"labix.org/v2/mgo"
+
+	"launchpad.net/juju-core/state/api/params"
 )
 
 // Life represents the lifecycle state of the entities
@@ -15,14 +20,14 @@ const (
 	nLife
 )
 
-var lifeStrings = [nLife]string{
-	Alive: "alive",
-	Dying: "dying",
-	Dead:  "dead",
+var lifeStrings = [nLife]params.Life{
+	Alive: params.Alive,
+	Dying: params.Dying,
+	Dead:  params.Dead,
 }
 
 func (l Life) String() string {
-	return lifeStrings[l]
+	return string(lifeStrings[l])
 }
 
 var isAliveDoc = D{{"life", Alive}}
