@@ -1,20 +1,27 @@
+// Copyright 2012, 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package state
 
 import (
 	"fmt"
+	"strings"
+	"time"
+
 	"labix.org/v2/mgo"
 	"labix.org/v2/mgo/txn"
+
 	"launchpad.net/juju-core/constraints"
+	"launchpad.net/juju-core/errors"
+	"launchpad.net/juju-core/instance"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/presence"
+	"launchpad.net/juju-core/tools"
 	"launchpad.net/juju-core/utils"
-	"time"
+	"launchpad.net/juju-core/version"
 )
 
-// An InstanceId is a provider-specific identifier associated with an
-// instance (physical or virtual machine allocated in the provider).
-type InstanceId string
-
 // Machine represents the state of a machine.
 type Machine struct {
 	st  *State
@@ -30,36 +37,52 @@ const (
 	_ MachineJob = iota
 	JobHostUnits
 	JobManageEnviron
-	JobServeAPI
+	JobManageState
 )
 
-var jobNames = []string{
-	JobHostUnits:     "JobHostUnits",
-	JobManageEnviron: "JobManageEnviron",
-	JobServeAPI:      "JobServeAPI",
+var jobNames = map[MachineJob]params.MachineJob{
+	JobHostUnits:     params.JobHostUnits,
+	JobManageEnviron: params.JobManageEnviron,
+	JobManageState:   params.JobManageState,
+}
+
+// AllJobs returns all supported machine jobs.
+func AllJobs() []MachineJob {
+	return []MachineJob{JobHostUnits, JobManageState, JobManageEnviron}
+}
+
+// ToParams returns the job as params.MachineJob.
+func (job MachineJob) ToParams() params.MachineJob {
+	if paramsJob, ok := jobNames[job]; ok {
+		return paramsJob
+	}
+	return params.MachineJob(fmt.Sprintf("<unknown job %d>", int(job)))
 }
 
 func (job MachineJob) String() string {
-	j := int(job)
-	if j <= 0 || j >= len(jobNames) {
-		return fmt.Sprintf("<unknown job %d>", j)
-	}
-	return jobNames[j]
+	return string(job.ToParams())
 }
 
 // machineDoc represents the internal state of a machine in MongoDB.
 // Note the correspondence with MachineInfo in state/api/params.
 type machineDoc struct {
-	Id           string `bson:"_id"`
-	Nonce        string
-	Series       string
-	InstanceId   InstanceId
-	Principals   []string
-	Life         Life
-	Tools        *Tools `bson:",omitempty"`
-	TxnRevno     int64  `bson:"txn-revno"`
-	Jobs         []MachineJob
-	PasswordHash string
+	Id            string `bson:"_id"`
+	Nonce         string
+	Series        string
+	ContainerType string
+	Principals    []string
+	Life          Life
+	Tools         *tools.Tools `bson:",omitempty"`
+	TxnRevno      int64        `bson:"txn-revno"`
+	Jobs          []MachineJob
+	PasswordHash  string
+	Clean         bool
+	Addresses     []address
+	// Deprecated. InstanceId, now lives on instanceData.
+	// This attribute is retained so that data from existing machines can be read.
+	// SCHEMACHANGE
+	// TODO(wallyworld): remove this attribute when schema upgrades are possible.
+	InstanceId instance.Id
 }
 
 func newMachine(st *State, doc *machineDoc) *Machine {
@@ -85,6 +108,11 @@ func (m *Machine) Series() string {
 	return m.doc.Series
 }
 
+// ContainerType returns the type of container hosting this machine.
+func (m *Machine) ContainerType() instance.ContainerType {
+	return instance.ContainerType(m.doc.ContainerType)
+}
+
 // machineGlobalKey returns the global database key for the identified machine.
 func machineGlobalKey(id string) string {
 	return "m#" + id
@@ -95,17 +123,52 @@ func (m *Machine) globalKey() string {
 	return machineGlobalKey(m.doc.Id)
 }
 
-// MachineTag returns the tag for the
-// machine with the given id.
-func MachineTag(id string) string {
-	return fmt.Sprintf("machine-%s", id)
+// instanceData holds attributes relevant to a provisioned machine.
+type instanceData struct {
+	Id         string      `bson:"_id"`
+	InstanceId instance.Id `bson:"instanceid"`
+	Arch       *string     `bson:"arch,omitempty"`
+	Mem        *uint64     `bson:"mem,omitempty"`
+	RootDisk   *uint64     `bson:"rootdisk,omitempty"`
+	CpuCores   *uint64     `bson:"cpucores,omitempty"`
+	CpuPower   *uint64     `bson:"cpupower,omitempty"`
+	Tags       *[]string   `bson:"tags,omitempty"`
+	TxnRevno   int64       `bson:"txn-revno"`
+}
+
+// TODO(wallyworld): move this method to a service.
+func (m *Machine) HardwareCharacteristics() (*instance.HardwareCharacteristics, error) {
+	hc := &instance.HardwareCharacteristics{}
+	instData, err := getInstanceData(m.st, m.Id())
+	if err != nil {
+		return nil, err
+	}
+	hc.Arch = instData.Arch
+	hc.Mem = instData.Mem
+	hc.RootDisk = instData.RootDisk
+	hc.CpuCores = instData.CpuCores
+	hc.CpuPower = instData.CpuPower
+	hc.Tags = instData.Tags
+	return hc, nil
+}
+
+func getInstanceData(st *State, id string) (instanceData, error) {
+	var instData instanceData
+	err := st.instanceData.FindId(id).One(&instData)
+	if err == mgo.ErrNotFound {
+		return instanceData{}, errors.NotFoundf("instance data for machine %v", id)
+	}
+	if err != nil {
+		return instanceData{}, fmt.Errorf("cannot get instance data for machine %v: %v", id, err)
+	}
+	return instData, nil
 }
 
 // Tag returns a name identifying the machine that is safe to use
 // as a file name.  The returned name will be different from other
 // Tag values returned by any other entities from the same state.
 func (m *Machine) Tag() string {
-	return MachineTag(m.Id())
+	return names.MachineTag(m.Id())
 }
 
 // Life returns whether the machine is Alive, Dying or Dead.
@@ -118,33 +181,53 @@ func (m *Machine) Jobs() []MachineJob {
 	return m.doc.Jobs
 }
 
+// IsStateServer returns true if the machine has a JobManageState job.
+func (m *Machine) IsStateServer() bool {
+	for _, job := range m.doc.Jobs {
+		if job == JobManageState {
+			return true
+		}
+	}
+	return false
+}
+
 // AgentTools returns the tools that the agent is currently running.
 // It returns an error that satisfies IsNotFound if the tools have not yet been set.
-func (m *Machine) AgentTools() (*Tools, error) {
+func (m *Machine) AgentTools() (*tools.Tools, error) {
 	if m.doc.Tools == nil {
-		return nil, NotFoundf("agent tools for machine %v", m)
+		return nil, errors.NotFoundf("agent tools for machine %v", m)
 	}
 	tools := *m.doc.Tools
 	return &tools, nil
 }
 
-// SetAgentTools sets the tools that the agent is currently running.
-func (m *Machine) SetAgentTools(t *Tools) (err error) {
-	defer utils.ErrorContextf(&err, "cannot set agent tools for machine %v", m)
-	if t.Series == "" || t.Arch == "" {
+// checkVersionValidity checks whether the given version is suitable
+// for passing to SetAgentVersion.
+func checkVersionValidity(v version.Binary) error {
+	if v.Series == "" || v.Arch == "" {
 		return fmt.Errorf("empty series or arch")
 	}
+	return nil
+}
+
+// SetAgentVersion sets the version of juju that the agent is
+// currently running.
+func (m *Machine) SetAgentVersion(v version.Binary) (err error) {
+	defer utils.ErrorContextf(&err, "cannot set agent version for machine %v", m)
+	if err = checkVersionValidity(v); err != nil {
+		return err
+	}
+	tools := &tools.Tools{Version: v}
 	ops := []txn.Op{{
 		C:      m.st.machines.Name,
 		Id:     m.doc.Id,
 		Assert: notDeadDoc,
-		Update: D{{"$set", D{{"tools", t}}}},
+		Update: D{{"$set", D{{"tools", tools}}}},
 	}}
-	if err := m.st.runner.Run(ops, "", nil); err != nil {
+	if err := m.st.runTransaction(ops); err != nil {
 		return onAbort(err, errDead)
 	}
-	tools := *t
-	m.doc.Tools = &tools
+	m.doc.Tools = tools
 	return nil
 }
 
@@ -164,7 +247,7 @@ func (m *Machine) SetPassword(password string) error {
 		Assert: notDeadDoc,
 		Update: D{{"$set", D{{"passwordhash", hp}}}},
 	}}
-	if err := m.st.runner.Run(ops, "", nil); err != nil {
+	if err := m.st.runTransaction(ops); err != nil {
 		return fmt.Errorf("cannot set password of machine %v: %v", m, onAbort(err, errDead))
 	}
 	m.doc.PasswordHash = hp
@@ -204,12 +287,61 @@ func (e *HasAssignedUnitsError) Error() string {
 	return fmt.Sprintf("machine %s has unit %q assigned", e.MachineId, e.UnitNames[0])
 }
 
+func IsHasAssignedUnitsError(err error) bool {
+	_, ok := err.(*HasAssignedUnitsError)
+	return ok
+}
+
+// Containers returns the container ids belonging to a parent machine.
+// TODO(wallyworld): move this method to a service
+func (m *Machine) Containers() ([]string, error) {
+	var mc machineContainers
+	err := m.st.containerRefs.FindId(m.Id()).One(&mc)
+	if err == nil {
+		return mc.Children, nil
+	}
+	if err == mgo.ErrNotFound {
+		return nil, errors.NotFoundf("container info for machine %v", m.Id())
+	}
+	return nil, err
+}
+
+// ParentId returns the Id of the host machine if this machine is a container.
+func (m *Machine) ParentId() (string, bool) {
+	parentId := ParentId(m.Id())
+	return parentId, parentId != ""
+}
+
+type HasContainersError struct {
+	MachineId    string
+	ContainerIds []string
+}
+
+func (e *HasContainersError) Error() string {
+	return fmt.Sprintf("machine %s is hosting containers %q", e.MachineId, strings.Join(e.ContainerIds, ","))
+}
+
+func IsHasContainersError(err error) bool {
+	_, ok := err.(*HasContainersError)
+	return ok
+}
+
 // advanceLifecycle ensures that the machine's lifecycle is no earlier
 // than the supplied value. If the machine already has that lifecycle
 // value, or a later one, no changes will be made to remote state. If
 // the machine has any responsibilities that preclude a valid change in
 // lifecycle, it will return an error.
 func (original *Machine) advanceLifecycle(life Life) (err error) {
+	containers, err := original.Containers()
+	if err != nil {
+		return err
+	}
+	if len(containers) > 0 {
+		return &HasContainersError{
+			MachineId:    original.doc.Id,
+			ContainerIds: containers,
+		}
+	}
 	m := original
 	defer func() {
 		if err == nil {
@@ -237,7 +369,7 @@ func (original *Machine) advanceLifecycle(life Life) (err error) {
 			{{"principals", D{{"$exists", false}}}},
 		}},
 	}
-	// 3 atempts: one with original data, one with refreshed data, and a final
+	// 3 attempts: one with original data, one with refreshed data, and a final
 	// one intended to determine the cause of failure of the preceding attempt.
 	for i := 0; i < 3; i++ {
 		// If the transaction was aborted, grab a fresh copy of the machine data.
@@ -247,7 +379,7 @@ func (original *Machine) advanceLifecycle(life Life) (err error) {
 		// context of the new state API, but we maintain consistency in the
 		// face of uncertainty.
 		if i != 0 {
-			if m, err = m.st.Machine(m.doc.Id); IsNotFound(err) {
+			if m, err = m.st.Machine(m.doc.Id); errors.IsNotFoundError(err) {
 				return nil
 			} else if err != nil {
 				return err
@@ -286,7 +418,7 @@ func (original *Machine) advanceLifecycle(life Life) (err error) {
 			}
 		}
 		// Run the transaction...
-		if err := m.st.runner.Run([]txn.Op{op}, "", nil); err != txn.ErrAborted {
+		if err := m.st.runTransaction([]txn.Op{op}); err != txn.ErrAborted {
 			return err
 		}
 		// ...and retry on abort.
@@ -312,13 +444,19 @@ func (m *Machine) Remove() (err error) {
 			Assert: txn.DocExists,
 			Remove: true,
 		},
+		{
+			C:      m.st.instanceData.Name,
+			Id:     m.doc.Id,
+			Remove: true,
+		},
 		removeStatusOp(m.st, m.globalKey()),
 		removeConstraintsOp(m.st, m.globalKey()),
 		annotationRemoveOp(m.st, m.globalKey()),
 	}
+	ops = append(ops, removeContainerRefOps(m.st, m.Id())...)
 	// The only abort conditions in play indicate that the machine has already
 	// been removed.
-	return onAbort(m.st.runner.Run(ops, "", nil), nil)
+	return onAbort(m.st.runTransaction(ops), nil)
 }
 
 // Refresh refreshes the contents of the machine from the underlying
@@ -328,7 +466,7 @@ func (m *Machine) Refresh() error {
 	doc := machineDoc{}
 	err := m.st.machines.FindId(m.doc.Id).One(&doc)
 	if err == mgo.ErrNotFound {
-		return NotFoundf("machine %v", m)
+		return errors.NotFoundf("machine %v", m)
 	}
 	if err != nil {
 		return fmt.Errorf("cannot refresh machine %v: %v", m, err)
@@ -374,10 +512,23 @@ func (m *Machine) SetAgentAlive() (*presence.Pinger, error) {
 	return p, nil
 }
 
-// InstanceId returns the provider specific instance id for this machine
-// and whether it has been set.
-func (m *Machine) InstanceId() (InstanceId, bool) {
-	return m.doc.InstanceId, m.doc.InstanceId != ""
+// InstanceId returns the provider specific instance id for this
+// machine, or a NotProvisionedError, if not set.
+func (m *Machine) InstanceId() (instance.Id, error) {
+	// SCHEMACHANGE
+	// TODO(wallyworld) - remove this backward compatibility code when schema upgrades are possible
+	// (we first check for InstanceId stored on the machineDoc)
+	if m.doc.InstanceId != "" {
+		return m.doc.InstanceId, nil
+	}
+	instData, err := getInstanceData(m.st, m.Id())
+	if (err == nil && instData.InstanceId == "") || (err != nil && errors.IsNotFoundError(err)) {
+		err = NotProvisionedError(m.Id())
+	}
+	if err != nil {
+		return "", err
+	}
+	return instData.InstanceId, nil
 }
 
 // Units returns all the units that have been assigned to the machine.
@@ -402,25 +553,51 @@ func (m *Machine) Units() (units []*Unit, err error) {
 	return units, nil
 }
 
-// SetProvisioned sets the provider specific machine id and nonce for
+// SetProvisioned sets the provider specific machine id, nonce and also metadata for
 // this machine. Once set, the instance id cannot be changed.
-func (m *Machine) SetProvisioned(id InstanceId, nonce string) (err error) {
-	defer utils.ErrorContextf(&err, "cannot set instance id of machine %q", m)
+func (m *Machine) SetProvisioned(id instance.Id, nonce string, characteristics *instance.HardwareCharacteristics) (err error) {
+	defer utils.ErrorContextf(&err, "cannot set instance data for machine %q", m)
 
 	if id == "" || nonce == "" {
 		return fmt.Errorf("instance id and nonce cannot be empty")
 	}
-	notSetYet := D{{"instanceid", ""}, {"nonce", ""}}
-	ops := []txn.Op{{
-		C:      m.st.machines.Name,
-		Id:     m.doc.Id,
-		Assert: append(isAliveDoc, notSetYet...),
-		Update: D{{"$set", D{{"instanceid", id}, {"nonce", nonce}}}},
-	}}
 
-	if err = m.st.runner.Run(ops, "", nil); err == nil {
-		m.doc.InstanceId = id
+	if characteristics == nil {
+		characteristics = &instance.HardwareCharacteristics{}
+	}
+	hc := &instanceData{
+		Id:         m.doc.Id,
+		InstanceId: id,
+		Arch:       characteristics.Arch,
+		Mem:        characteristics.Mem,
+		RootDisk:   characteristics.RootDisk,
+		CpuCores:   characteristics.CpuCores,
+		CpuPower:   characteristics.CpuPower,
+		Tags:       characteristics.Tags,
+	}
+	// SCHEMACHANGE
+	// TODO(wallyworld) - do not check instanceId on machineDoc after schema is upgraded
+	notSetYet := D{{"instanceid", ""}, {"nonce", ""}}
+	ops := []txn.Op{
+		{
+			C:      m.st.machines.Name,
+			Id:     m.doc.Id,
+			Assert: append(isAliveDoc, notSetYet...),
+			Update: D{{"$set", D{{"instanceid", id}, {"nonce", nonce}}}},
+		}, {
+			C:      m.st.instanceData.Name,
+			Id:     m.doc.Id,
+			Assert: txn.DocMissing,
+			Insert: hc,
+		},
+	}
+
+	if err = m.st.runTransaction(ops); err == nil {
 		m.doc.Nonce = nonce
+		// SCHEMACHANGE
+		// TODO(wallyworld) - remove this backward compatibility code when schema upgrades are possible
+		// (InstanceId is stored on the instanceData document but we duplicate the value on the machineDoc.
+		m.doc.InstanceId = id
 		return nil
 	} else if err != txn.ErrAborted {
 		return err
@@ -432,10 +609,59 @@ func (m *Machine) SetProvisioned(id InstanceId, nonce string) (err error) {
 	return fmt.Errorf("already set")
 }
 
-// CheckProvisioned returns true if the machine was provisioned with
-// the given nonce.
+// notProvisionedError records an error when a machine is not provisioned.
+type notProvisionedError struct {
+	machineId string
+}
+
+func NotProvisionedError(machineId string) error {
+	return &notProvisionedError{machineId}
+}
+
+func (e *notProvisionedError) Error() string {
+	return fmt.Sprintf("machine %v is not provisioned", e.machineId)
+}
+
+// IsNotProvisionedError returns true if err is a notProvisionedError.
+func IsNotProvisionedError(err error) bool {
+	_, ok := err.(*notProvisionedError)
+	return ok
+}
+
+// Addresses returns any hostnames and ips associated with a machine
+func (m *Machine) Addresses() (addresses []instance.Address) {
+	for _, address := range m.doc.Addresses {
+		addresses = append(addresses, address.InstanceAddress())
+	}
+	return
+}
+
+// SetAddresses records any addresses related to the machine
+func (m *Machine) SetAddresses(addresses []instance.Address) (err error) {
+	var stateAddresses []address
+	for _, address := range addresses {
+		stateAddresses = append(stateAddresses, NewAddress(address))
+	}
+
+	ops := []txn.Op{
+		{
+			C:      m.st.machines.Name,
+			Id:     m.doc.Id,
+			Assert: notDeadDoc,
+			Update: D{{"$set", D{{"addresses", stateAddresses}}}},
+		},
+	}
+
+	if err = m.st.runTransaction(ops); err != nil {
+		return fmt.Errorf("cannot set addresses of machine %v: %v", m, onAbort(err, errDead))
+	}
+	m.doc.Addresses = stateAddresses
+	return nil
+}
+
+// CheckProvisioned returns true if the machine was provisioned with the given nonce.
 func (m *Machine) CheckProvisioned(nonce string) bool {
-	return m.doc.Nonce == nonce && m.doc.InstanceId != ""
+	return nonce == m.doc.Nonce && nonce != ""
 }
 
 // String returns a unique description of this machine.
@@ -454,12 +680,12 @@ func (m *Machine) Constraints() (constraints.Value, error) {
 // is already provisioned.
 func (m *Machine) SetConstraints(cons constraints.Value) (err error) {
 	defer utils.ErrorContextf(&err, "cannot set constraints")
-	notProvisioned := D{{"instanceid", ""}}
+	notSetYet := D{{"nonce", ""}}
 	ops := []txn.Op{
 		{
 			C:      m.st.machines.Name,
 			Id:     m.doc.Id,
-			Assert: append(isAliveDoc, notProvisioned...),
+			Assert: append(isAliveDoc, notSetYet...),
 		},
 		setConstraintsOp(m.st, m.globalKey(), cons),
 	}
@@ -472,10 +698,12 @@ func (m *Machine) SetConstraints(cons constraints.Value) (err error) {
 		if m.doc.Life != Alive {
 			return errNotAlive
 		}
-		if m.doc.InstanceId != "" {
+		if _, err := m.InstanceId(); err == nil {
 			return fmt.Errorf("machine is already provisioned")
+		} else if !IsNotProvisionedError(err) {
+			return err
 		}
-		if err := m.st.runner.Run(ops, "", nil); err != txn.ErrAborted {
+		if err := m.st.runTransaction(ops); err != txn.ErrAborted {
 			return err
 		}
 		if m, err = m.st.Machine(m.doc.Id); err != nil {
@@ -486,27 +714,26 @@ func (m *Machine) SetConstraints(cons constraints.Value) (err error) {
 }
 
 // Status returns the status of the machine.
-func (m *Machine) Status() (status params.Status, info string, err error) {
+func (m *Machine) Status() (status params.Status, info string, data params.StatusData, err error) {
 	doc, err := getStatus(m.st, m.globalKey())
 	if err != nil {
-		return "", "", err
+		return "", "", nil, err
 	}
 	status = doc.Status
 	info = doc.StatusInfo
+	data = doc.StatusData
 	return
 }
 
 // SetStatus sets the status of the machine.
-func (m *Machine) SetStatus(status params.Status, info string) error {
-	if status == params.StatusError && info == "" {
-		panic("machine error status with no info")
-	}
-	if status == params.StatusPending {
-		panic("machine status cannot be set to pending")
-	}
+func (m *Machine) SetStatus(status params.Status, info string, data params.StatusData) error {
 	doc := statusDoc{
 		Status:     status,
 		StatusInfo: info,
+		StatusData: data,
+	}
+	if err := doc.validateSet(); err != nil {
+		return err
 	}
 	ops := []txn.Op{{
 		C:      m.st.machines.Name,
@@ -515,8 +742,13 @@ func (m *Machine) SetStatus(status params.Status, info string) error {
 	},
 		updateStatusOp(m.st, m.globalKey(), doc),
 	}
-	if err := m.st.runner.Run(ops, "", nil); err != nil {
+	if err := m.st.runTransaction(ops); err != nil {
 		return fmt.Errorf("cannot set status of machine %q: %v", m, onAbort(err, errNotAlive))
 	}
 	return nil
 }
+
+// Clean returns true if the machine does not have any deployed units or containers.
+func (m *Machine) Clean() bool {
+	return m.doc.Clean
+}
