@@ -1,68 +1,80 @@
+// Copyright 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package state
 
 import (
 	"errors"
 	"fmt"
+	"reflect"
+	"sort"
+	"time"
+
 	"labix.org/v2/mgo"
-	. "launchpad.net/gocheck"
+	gc "launchpad.net/gocheck"
+
 	"launchpad.net/juju-core/charm"
 	"launchpad.net/juju-core/constraints"
+	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/multiwatcher"
 	"launchpad.net/juju-core/state/watcher"
 	"launchpad.net/juju-core/testing"
-	"reflect"
-	"sort"
-	"time"
+	"launchpad.net/juju-core/testing/testbase"
 )
 
+var dottedConfig = `
+options:
+  key.dotted: {default: My Key, description: Desc, type: string}
+`
+
 type storeManagerStateSuite struct {
-	testing.LoggingSuite
+	testbase.LoggingSuite
 	testing.MgoSuite
 	State *State
 }
 
-func (s *storeManagerStateSuite) SetUpSuite(c *C) {
+func (s *storeManagerStateSuite) SetUpSuite(c *gc.C) {
 	s.LoggingSuite.SetUpSuite(c)
 	s.MgoSuite.SetUpSuite(c)
 }
 
-func (s *storeManagerStateSuite) TearDownSuite(c *C) {
+func (s *storeManagerStateSuite) TearDownSuite(c *gc.C) {
 	s.MgoSuite.TearDownSuite(c)
 	s.LoggingSuite.TearDownSuite(c)
 }
 
-func (s *storeManagerStateSuite) SetUpTest(c *C) {
+func (s *storeManagerStateSuite) SetUpTest(c *gc.C) {
 	s.LoggingSuite.SetUpTest(c)
 	s.MgoSuite.SetUpTest(c)
 	s.State = TestingInitialize(c, nil)
 }
 
-func (s *storeManagerStateSuite) TearDownTest(c *C) {
+func (s *storeManagerStateSuite) TearDownTest(c *gc.C) {
 	s.State.Close()
 	s.MgoSuite.TearDownTest(c)
 	s.LoggingSuite.TearDownTest(c)
 }
 
-func (s *storeManagerStateSuite) Reset(c *C) {
+func (s *storeManagerStateSuite) Reset(c *gc.C) {
 	s.TearDownTest(c)
 	s.SetUpTest(c)
 }
 
-var _ = Suite(&storeManagerStateSuite{})
+var _ = gc.Suite(&storeManagerStateSuite{})
 
 // setUpScenario adds some entities to the state so that
 // we can check that they all get pulled in by
 // allWatcherStateBacking.getAll.
-func (s *storeManagerStateSuite) setUpScenario(c *C) (entities entityInfoSlice) {
+func (s *storeManagerStateSuite) setUpScenario(c *gc.C) (entities entityInfoSlice) {
 	add := func(e params.EntityInfo) {
 		entities = append(entities, e)
 	}
-	m, err := s.State.AddMachine("series", JobManageEnviron)
-	c.Assert(err, IsNil)
-	c.Assert(m.Tag(), Equals, "machine-0")
-	err = m.SetProvisioned(InstanceId("i-"+m.Tag()), "fake_nonce")
-	c.Assert(err, IsNil)
+	m, err := s.State.AddMachine("quantal", JobManageEnviron)
+	c.Assert(err, gc.IsNil)
+	c.Assert(m.Tag(), gc.Equals, "machine-0")
+	err = m.SetProvisioned(instance.Id("i-"+m.Tag()), "fake_nonce", nil)
+	c.Assert(err, gc.IsNil)
 	add(&params.MachineInfo{
 		Id:         "0",
 		InstanceId: "i-machine-0",
@@ -70,77 +82,81 @@ func (s *storeManagerStateSuite) setUpScenario(c *C) (entities entityInfoSlice)
 	})
 
 	wordpress, err := s.State.AddService("wordpress", AddTestingCharm(c, s.State, "wordpress"))
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	err = wordpress.SetExposed()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
+	err = wordpress.SetMinUnits(3)
+	c.Assert(err, gc.IsNil)
 	err = wordpress.SetConstraints(constraints.MustParse("mem=100M"))
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	setServiceConfigAttr(c, wordpress, "blog-title", "boring")
 	add(&params.ServiceInfo{
 		Name:        "wordpress",
 		Exposed:     true,
 		CharmURL:    serviceCharmURL(wordpress).String(),
-		Life:        Alive.String(),
+		Life:        params.Life(Alive.String()),
+		MinUnits:    3,
 		Constraints: constraints.MustParse("mem=100M"),
-		Config:      map[string]interface{}{"blog-title": "boring"},
+		Config:      charm.Settings{"blog-title": "boring"},
 	})
 	pairs := map[string]string{"x": "12", "y": "99"}
 	err = wordpress.SetAnnotations(pairs)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	add(&params.AnnotationInfo{
 		Tag:         "service-wordpress",
 		Annotations: pairs,
 	})
 
 	logging, err := s.State.AddService("logging", AddTestingCharm(c, s.State, "logging"))
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	add(&params.ServiceInfo{
 		Name:     "logging",
 		CharmURL: serviceCharmURL(logging).String(),
-		Life:     Alive.String(),
-		Config:   map[string]interface{}{},
+		Life:     params.Life(Alive.String()),
+		Config:   charm.Settings{},
 	})
 
 	eps, err := s.State.InferEndpoints([]string{"logging", "wordpress"})
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	rel, err := s.State.AddRelation(eps...)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	add(&params.RelationInfo{
 		Key: "logging:logging-directory wordpress:logging-dir",
+		Id:  rel.Id(),
 		Endpoints: []params.Endpoint{
-			params.Endpoint{ServiceName: "logging", Relation: charm.Relation{Name: "logging-directory", Role: "requirer", Interface: "logging", Optional: false, Limit: 1, Scope: "container"}},
-			params.Endpoint{ServiceName: "wordpress", Relation: charm.Relation{Name: "logging-dir", Role: "provider", Interface: "logging", Optional: false, Limit: 0, Scope: "container"}}},
+			{ServiceName: "logging", Relation: charm.Relation{Name: "logging-directory", Role: "requirer", Interface: "logging", Optional: false, Limit: 1, Scope: "container"}},
+			{ServiceName: "wordpress", Relation: charm.Relation{Name: "logging-dir", Role: "provider", Interface: "logging", Optional: false, Limit: 0, Scope: "container"}}},
 	})
 
 	for i := 0; i < 2; i++ {
 		wu, err := wordpress.AddUnit()
-		c.Assert(err, IsNil)
-		c.Assert(wu.Tag(), Equals, fmt.Sprintf("unit-wordpress-%d", i))
+		c.Assert(err, gc.IsNil)
+		c.Assert(wu.Tag(), gc.Equals, fmt.Sprintf("unit-wordpress-%d", i))
 
-		m, err := s.State.AddMachine("series", JobHostUnits)
-		c.Assert(err, IsNil)
-		c.Assert(m.Tag(), Equals, fmt.Sprintf("machine-%d", i+1))
+		m, err := s.State.AddMachine("quantal", JobHostUnits)
+		c.Assert(err, gc.IsNil)
+		c.Assert(m.Tag(), gc.Equals, fmt.Sprintf("machine-%d", i+1))
 
 		add(&params.UnitInfo{
 			Name:      fmt.Sprintf("wordpress/%d", i),
 			Service:   wordpress.Name(),
 			Series:    m.Series(),
 			MachineId: m.Id(),
-			Ports:     []params.Port{},
+			Ports:     []instance.Port{},
 			Status:    params.StatusPending,
 		})
 		pairs := map[string]string{"name": fmt.Sprintf("bar %d", i)}
 		err = wu.SetAnnotations(pairs)
-		c.Assert(err, IsNil)
+		c.Assert(err, gc.IsNil)
 		add(&params.AnnotationInfo{
 			Tag:         fmt.Sprintf("unit-wordpress-%d", i),
 			Annotations: pairs,
 		})
 
-		err = m.SetProvisioned(InstanceId("i-"+m.Tag()), "fake_nonce")
-		c.Assert(err, IsNil)
-		err = m.SetStatus(params.StatusError, m.Tag())
-		c.Assert(err, IsNil)
+		err = m.SetProvisioned(instance.Id("i-"+m.Tag()), "fake_nonce", nil)
+		c.Assert(err, gc.IsNil)
+		err = m.SetStatus(params.StatusError, m.Tag(), nil)
+		c.Assert(err, gc.IsNil)
 		add(&params.MachineInfo{
 			Id:         fmt.Sprint(i + 1),
 			InstanceId: "i-" + m.Tag(),
@@ -148,31 +164,31 @@ func (s *storeManagerStateSuite) setUpScenario(c *C) (entities entityInfoSlice)
 			StatusInfo: m.Tag(),
 		})
 		err = wu.AssignToMachine(m)
-		c.Assert(err, IsNil)
+		c.Assert(err, gc.IsNil)
 
 		deployer, ok := wu.DeployerTag()
-		c.Assert(ok, Equals, true)
-		c.Assert(deployer, Equals, fmt.Sprintf("machine-%d", i+1))
+		c.Assert(ok, gc.Equals, true)
+		c.Assert(deployer, gc.Equals, fmt.Sprintf("machine-%d", i+1))
 
 		wru, err := rel.Unit(wu)
-		c.Assert(err, IsNil)
+		c.Assert(err, gc.IsNil)
 
 		// Create the subordinate unit as a side-effect of entering
 		// scope in the principal's relation-unit.
 		err = wru.EnterScope(nil)
-		c.Assert(err, IsNil)
+		c.Assert(err, gc.IsNil)
 
 		lu, err := s.State.Unit(fmt.Sprintf("logging/%d", i))
-		c.Assert(err, IsNil)
-		c.Assert(lu.IsPrincipal(), Equals, false)
+		c.Assert(err, gc.IsNil)
+		c.Assert(lu.IsPrincipal(), gc.Equals, false)
 		deployer, ok = lu.DeployerTag()
-		c.Assert(ok, Equals, true)
-		c.Assert(deployer, Equals, fmt.Sprintf("unit-wordpress-%d", i))
+		c.Assert(ok, gc.Equals, true)
+		c.Assert(deployer, gc.Equals, fmt.Sprintf("unit-wordpress-%d", i))
 		add(&params.UnitInfo{
 			Name:    fmt.Sprintf("logging/%d", i),
 			Service: "logging",
-			Series:  "series",
-			Ports:   []params.Port{},
+			Series:  "quantal",
+			Ports:   []instance.Port{},
 			Status:  params.StatusPending,
 		})
 	}
@@ -184,7 +200,7 @@ func serviceCharmURL(svc *Service) *charm.URL {
 	return url
 }
 
-func assertEntitiesEqual(c *C, got, want []params.EntityInfo) {
+func assertEntitiesEqual(c *gc.C, got, want []params.EntityInfo) {
 	if len(got) == 0 {
 		got = nil
 	}
@@ -206,12 +222,12 @@ func assertEntitiesEqual(c *C, got, want []params.EntityInfo) {
 	c.FailNow()
 }
 
-func (s *storeManagerStateSuite) TestStateBackingGetAll(c *C) {
+func (s *storeManagerStateSuite) TestStateBackingGetAll(c *gc.C) {
 	expectEntities := s.setUpScenario(c)
 	b := newAllWatcherStateBacking(s.State)
 	all := multiwatcher.NewStore()
 	err := b.GetAll(all)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	var gotEntities entityInfoSlice = all.All()
 	sort.Sort(gotEntities)
 	sort.Sort(expectEntities)
@@ -221,14 +237,14 @@ func (s *storeManagerStateSuite) TestStateBackingGetAll(c *C) {
 var allWatcherChangedTests = []struct {
 	about          string
 	add            []params.EntityInfo
-	setUp          func(c *C, st *State)
+	setUp          func(c *gc.C, st *State)
 	change         watcher.Change
 	expectContents []params.EntityInfo
 }{
 	// Machine changes
 	{
 		about: "no machine in state, no machine in store -> do nothing",
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "machines",
 			Id: "1",
@@ -236,18 +252,18 @@ var allWatcherChangedTests = []struct {
 	}, {
 		about: "machine is removed if it's not in backing",
 		add:   []params.EntityInfo{&params.MachineInfo{Id: "1"}},
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "machines",
 			Id: "1",
 		},
 	}, {
 		about: "machine is added if it's in backing but not in Store",
-		setUp: func(c *C, st *State) {
-			m, err := st.AddMachine("series", JobHostUnits)
-			c.Assert(err, IsNil)
-			err = m.SetStatus(params.StatusError, "failure")
-			c.Assert(err, IsNil)
+		setUp: func(c *gc.C, st *State) {
+			m, err := st.AddMachine("quantal", JobHostUnits)
+			c.Assert(err, gc.IsNil)
+			err = m.SetStatus(params.StatusError, "failure", nil)
+			c.Assert(err, gc.IsNil)
 		},
 		change: watcher.Change{
 			C:  "machines",
@@ -260,7 +276,9 @@ var allWatcherChangedTests = []struct {
 				StatusInfo: "failure",
 			},
 		},
-	}, {
+	},
+	// Machine status changes
+	{
 		about: "machine is updated if it's in backing and in Store",
 		add: []params.EntityInfo{
 			&params.MachineInfo{
@@ -269,11 +287,11 @@ var allWatcherChangedTests = []struct {
 				StatusInfo: "another failure",
 			},
 		},
-		setUp: func(c *C, st *State) {
-			m, err := st.AddMachine("series", JobManageEnviron)
-			c.Assert(err, IsNil)
-			err = m.SetProvisioned("i-0", "bootstrap_nonce")
-			c.Assert(err, IsNil)
+		setUp: func(c *gc.C, st *State) {
+			m, err := st.AddMachine("quantal", JobManageEnviron)
+			c.Assert(err, gc.IsNil)
+			err = m.SetProvisioned("i-0", "bootstrap_nonce", nil)
+			c.Assert(err, gc.IsNil)
 		},
 		change: watcher.Change{
 			C:  "machines",
@@ -291,7 +309,7 @@ var allWatcherChangedTests = []struct {
 	// Unit changes
 	{
 		about: "no unit in state, no unit in store -> do nothing",
-		setUp: func(c *C, st *State) {},
+		setUp: func(c *gc.C, st *State) {},
 		change: watcher.Change{
 			C:  "units",
 			Id: "1",
@@ -299,30 +317,30 @@ var allWatcherChangedTests = []struct {
 	}, {
 		about: "unit is removed if it's not in backing",
 		add:   []params.EntityInfo{&params.UnitInfo{Name: "wordpress/1"}},
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "units",
 			Id: "wordpress/1",
 		},
 	}, {
 		about: "unit is added if it's in backing but not in Store",
-		setUp: func(c *C, st *State) {
+		setUp: func(c *gc.C, st *State) {
 			wordpress, err := st.AddService("wordpress", AddTestingCharm(c, st, "wordpress"))
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			u, err := wordpress.AddUnit()
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			err = u.SetPublicAddress("public")
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			err = u.SetPrivateAddress("private")
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			err = u.OpenPort("tcp", 12345)
-			c.Assert(err, IsNil)
-			m, err := st.AddMachine("series", JobHostUnits)
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
+			m, err := st.AddMachine("quantal", JobHostUnits)
+			c.Assert(err, gc.IsNil)
 			err = u.AssignToMachine(m)
-			c.Assert(err, IsNil)
-			err = u.SetStatus(params.StatusError, "failure")
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
+			err = u.SetStatus(params.StatusError, "failure", nil)
+			c.Assert(err, gc.IsNil)
 		},
 		change: watcher.Change{
 			C:  "units",
@@ -332,11 +350,11 @@ var allWatcherChangedTests = []struct {
 			&params.UnitInfo{
 				Name:           "wordpress/0",
 				Service:        "wordpress",
-				Series:         "series",
+				Series:         "quantal",
 				PublicAddress:  "public",
 				PrivateAddress: "private",
 				MachineId:      "0",
-				Ports:          []params.Port{{"tcp", 12345}},
+				Ports:          []instance.Port{{"tcp", 12345}},
 				Status:         params.StatusError,
 				StatusInfo:     "failure",
 			},
@@ -348,15 +366,15 @@ var allWatcherChangedTests = []struct {
 			Status:     params.StatusError,
 			StatusInfo: "another failure",
 		}},
-		setUp: func(c *C, st *State) {
+		setUp: func(c *gc.C, st *State) {
 			wordpress, err := st.AddService("wordpress", AddTestingCharm(c, st, "wordpress"))
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			u, err := wordpress.AddUnit()
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			err = u.SetPublicAddress("public")
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			err = u.OpenPort("udp", 17070)
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 		},
 		change: watcher.Change{
 			C:  "units",
@@ -366,9 +384,9 @@ var allWatcherChangedTests = []struct {
 			&params.UnitInfo{
 				Name:          "wordpress/0",
 				Service:       "wordpress",
-				Series:        "series",
+				Series:        "quantal",
 				PublicAddress: "public",
-				Ports:         []params.Port{{"udp", 17070}},
+				Ports:         []instance.Port{{"udp", 17070}},
 				Status:        params.StatusError,
 				StatusInfo:    "another failure",
 			},
@@ -377,7 +395,7 @@ var allWatcherChangedTests = []struct {
 	// Service changes
 	{
 		about: "no service in state, no service in store -> do nothing",
-		setUp: func(c *C, st *State) {},
+		setUp: func(c *gc.C, st *State) {},
 		change: watcher.Change{
 			C:  "services",
 			Id: "wordpress",
@@ -385,18 +403,20 @@ var allWatcherChangedTests = []struct {
 	}, {
 		about: "service is removed if it's not in backing",
 		add:   []params.EntityInfo{&params.ServiceInfo{Name: "wordpress"}},
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "services",
 			Id: "wordpress",
 		},
 	}, {
 		about: "service is added if it's in backing but not in Store",
-		setUp: func(c *C, st *State) {
+		setUp: func(c *gc.C, st *State) {
 			wordpress, err := st.AddService("wordpress", AddTestingCharm(c, st, "wordpress"))
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			err = wordpress.SetExposed()
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
+			err = wordpress.SetMinUnits(42)
+			c.Assert(err, gc.IsNil)
 		},
 		change: watcher.Change{
 			C:  "services",
@@ -406,9 +426,10 @@ var allWatcherChangedTests = []struct {
 			&params.ServiceInfo{
 				Name:     "wordpress",
 				Exposed:  true,
-				CharmURL: "local:series/series-wordpress-3",
-				Life:     Alive.String(),
-				Config:   map[string]interface{}{},
+				CharmURL: "local:quantal/quantal-wordpress-3",
+				Life:     params.Life(Alive.String()),
+				MinUnits: 42,
+				Config:   charm.Settings{},
 			},
 		},
 	}, {
@@ -416,13 +437,14 @@ var allWatcherChangedTests = []struct {
 		add: []params.EntityInfo{&params.ServiceInfo{
 			Name:        "wordpress",
 			Exposed:     true,
-			CharmURL:    "local:series/series-wordpress-3",
+			CharmURL:    "local:quantal/quantal-wordpress-3",
+			MinUnits:    47,
 			Constraints: constraints.MustParse("mem=99M"),
-			Config:      map[string]interface{}{"blog-title": "boring"},
+			Config:      charm.Settings{"blog-title": "boring"},
 		}},
-		setUp: func(c *C, st *State) {
+		setUp: func(c *gc.C, st *State) {
 			svc, err := st.AddService("wordpress", AddTestingCharm(c, st, "wordpress"))
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			setServiceConfigAttr(c, svc, "blog-title", "boring")
 		},
 		change: watcher.Change{
@@ -432,10 +454,10 @@ var allWatcherChangedTests = []struct {
 		expectContents: []params.EntityInfo{
 			&params.ServiceInfo{
 				Name:        "wordpress",
-				CharmURL:    "local:series/series-wordpress-3",
-				Life:        Alive.String(),
+				CharmURL:    "local:quantal/quantal-wordpress-3",
+				Life:        params.Life(Alive.String()),
 				Constraints: constraints.MustParse("mem=99M"),
-				Config:      map[string]interface{}{"blog-title": "boring"},
+				Config:      charm.Settings{"blog-title": "boring"},
 			},
 		},
 	}, {
@@ -444,17 +466,13 @@ var allWatcherChangedTests = []struct {
 			Name: "wordpress",
 			// Note: CharmURL has a different revision number from
 			// the wordpress revision in the testing repo.
-			CharmURL: "local:series/series-wordpress-2",
-			Config:   map[string]interface{}{"foo": "bar"},
+			CharmURL: "local:quantal/quantal-wordpress-2",
+			Config:   charm.Settings{"foo": "bar"},
 		}},
-		setUp: func(c *C, st *State) {
+		setUp: func(c *gc.C, st *State) {
 			svc, err := st.AddService("wordpress", AddTestingCharm(c, st, "wordpress"))
-			c.Assert(err, IsNil)
-			cfg, err := svc.Config()
-			c.Assert(err, IsNil)
-			cfg.Set("blog-title", "boring")
-			_, err = cfg.Write()
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
+			setServiceConfigAttr(c, svc, "blog-title", "boring")
 		},
 		change: watcher.Change{
 			C:  "services",
@@ -463,16 +481,16 @@ var allWatcherChangedTests = []struct {
 		expectContents: []params.EntityInfo{
 			&params.ServiceInfo{
 				Name:     "wordpress",
-				CharmURL: "local:series/series-wordpress-3",
-				Life:     Alive.String(),
-				Config:   map[string]interface{}{"blog-title": "boring"},
+				CharmURL: "local:quantal/quantal-wordpress-3",
+				Life:     params.Life(Alive.String()),
+				Config:   charm.Settings{"blog-title": "boring"},
 			},
 		},
 	},
 	// Relation changes
 	{
 		about: "no relation in state, no service in store -> do nothing",
-		setUp: func(c *C, st *State) {},
+		setUp: func(c *gc.C, st *State) {},
 		change: watcher.Change{
 			C:  "relations",
 			Id: "logging:logging-directory wordpress:logging-dir",
@@ -480,23 +498,23 @@ var allWatcherChangedTests = []struct {
 	}, {
 		about: "relation is removed if it's not in backing",
 		add:   []params.EntityInfo{&params.RelationInfo{Key: "logging:logging-directory wordpress:logging-dir"}},
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "relations",
 			Id: "logging:logging-directory wordpress:logging-dir",
 		},
 	}, {
 		about: "relation is added if it's in backing but not in Store",
-		setUp: func(c *C, st *State) {
+		setUp: func(c *gc.C, st *State) {
 			_, err := st.AddService("wordpress", AddTestingCharm(c, st, "wordpress"))
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 
 			_, err = st.AddService("logging", AddTestingCharm(c, st, "logging"))
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			eps, err := st.InferEndpoints([]string{"logging", "wordpress"})
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			_, err = st.AddRelation(eps...)
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 		},
 		change: watcher.Change{
 			C:  "relations",
@@ -506,15 +524,15 @@ var allWatcherChangedTests = []struct {
 			&params.RelationInfo{
 				Key: "logging:logging-directory wordpress:logging-dir",
 				Endpoints: []params.Endpoint{
-					params.Endpoint{ServiceName: "logging", Relation: charm.Relation{Name: "logging-directory", Role: "requirer", Interface: "logging", Optional: false, Limit: 1, Scope: "container"}},
-					params.Endpoint{ServiceName: "wordpress", Relation: charm.Relation{Name: "logging-dir", Role: "provider", Interface: "logging", Optional: false, Limit: 0, Scope: "container"}}},
+					{ServiceName: "logging", Relation: charm.Relation{Name: "logging-directory", Role: "requirer", Interface: "logging", Optional: false, Limit: 1, Scope: "container"}},
+					{ServiceName: "wordpress", Relation: charm.Relation{Name: "logging-dir", Role: "provider", Interface: "logging", Optional: false, Limit: 0, Scope: "container"}}},
 			},
 		},
 	},
 	// Annotation changes
 	{
 		about: "no annotation in state, no annotation in store -> do nothing",
-		setUp: func(c *C, st *State) {},
+		setUp: func(c *gc.C, st *State) {},
 		change: watcher.Change{
 			C:  "relations",
 			Id: "m#0",
@@ -522,18 +540,18 @@ var allWatcherChangedTests = []struct {
 	}, {
 		about: "annotation is removed if it's not in backing",
 		add:   []params.EntityInfo{&params.AnnotationInfo{Tag: "machine-0"}},
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "annotations",
 			Id: "m#0",
 		},
 	}, {
 		about: "annotation is added if it's in backing but not in Store",
-		setUp: func(c *C, st *State) {
-			m, err := st.AddMachine("series", JobHostUnits)
-			c.Assert(err, IsNil)
+		setUp: func(c *gc.C, st *State) {
+			m, err := st.AddMachine("quantal", JobHostUnits)
+			c.Assert(err, gc.IsNil)
 			err = m.SetAnnotations(map[string]string{"foo": "bar", "arble": "baz"})
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 		},
 		change: watcher.Change{
 			C:  "annotations",
@@ -555,15 +573,15 @@ var allWatcherChangedTests = []struct {
 				"pretty": "polly",
 			},
 		}},
-		setUp: func(c *C, st *State) {
-			m, err := st.AddMachine("series", JobHostUnits)
-			c.Assert(err, IsNil)
+		setUp: func(c *gc.C, st *State) {
+			m, err := st.AddMachine("quantal", JobHostUnits)
+			c.Assert(err, gc.IsNil)
 			err = m.SetAnnotations(map[string]string{
 				"arble":  "khroomph",
 				"pretty": "",
 				"new":    "attr",
 			})
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 		},
 		change: watcher.Change{
 			C:  "annotations",
@@ -582,7 +600,7 @@ var allWatcherChangedTests = []struct {
 	// Unit status changes
 	{
 		about: "no unit in state -> do nothing",
-		setUp: func(c *C, st *State) {},
+		setUp: func(c *gc.C, st *State) {},
 		change: watcher.Change{
 			C:  "statuses",
 			Id: "u#wordpress/0",
@@ -594,7 +612,7 @@ var allWatcherChangedTests = []struct {
 			Status:     params.StatusError,
 			StatusInfo: "failure",
 		}},
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "statuses",
 			Id: "u#wordpress/0",
@@ -613,13 +631,13 @@ var allWatcherChangedTests = []struct {
 			Status:     params.StatusError,
 			StatusInfo: "failure",
 		}},
-		setUp: func(c *C, st *State) {
+		setUp: func(c *gc.C, st *State) {
 			wordpress, err := st.AddService("wordpress", AddTestingCharm(c, st, "wordpress"))
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			u, err := wordpress.AddUnit()
-			c.Assert(err, IsNil)
-			err = u.SetStatus(params.StatusStarted, "")
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
+			err = u.SetStatus(params.StatusStarted, "", nil)
+			c.Assert(err, gc.IsNil)
 		},
 		change: watcher.Change{
 			C:  "statuses",
@@ -627,15 +645,50 @@ var allWatcherChangedTests = []struct {
 		},
 		expectContents: []params.EntityInfo{
 			&params.UnitInfo{
-				Name:   "wordpress/0",
-				Status: params.StatusStarted,
+				Name:       "wordpress/0",
+				Status:     params.StatusStarted,
+				StatusData: params.StatusData{},
+			},
+		},
+	}, {
+		about: "status is changed with additional status data",
+		add: []params.EntityInfo{&params.UnitInfo{
+			Name:   "wordpress/0",
+			Status: params.StatusStarted,
+		}},
+		setUp: func(c *gc.C, st *State) {
+			wordpress, err := st.AddService("wordpress", AddTestingCharm(c, st, "wordpress"))
+			c.Assert(err, gc.IsNil)
+			u, err := wordpress.AddUnit()
+			c.Assert(err, gc.IsNil)
+			err = u.SetStatus(params.StatusError, "hook error", params.StatusData{
+				"1st-key": "one",
+				"2nd-key": 2,
+				"3rd-key": true,
+			})
+			c.Assert(err, gc.IsNil)
+		},
+		change: watcher.Change{
+			C:  "statuses",
+			Id: "u#wordpress/0",
+		},
+		expectContents: []params.EntityInfo{
+			&params.UnitInfo{
+				Name:       "wordpress/0",
+				Status:     params.StatusError,
+				StatusInfo: "hook error",
+				StatusData: params.StatusData{
+					"1st-key": "one",
+					"2nd-key": 2,
+					"3rd-key": true,
+				},
 			},
 		},
 	},
 	// Machine status changes
 	{
 		about: "no machine in state -> do nothing",
-		setUp: func(c *C, st *State) {},
+		setUp: func(c *gc.C, st *State) {},
 		change: watcher.Change{
 			C:  "statuses",
 			Id: "m#0",
@@ -647,7 +700,7 @@ var allWatcherChangedTests = []struct {
 			Status:     params.StatusError,
 			StatusInfo: "failure",
 		}},
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "statuses",
 			Id: "m#0",
@@ -664,11 +717,11 @@ var allWatcherChangedTests = []struct {
 			Status:     params.StatusError,
 			StatusInfo: "failure",
 		}},
-		setUp: func(c *C, st *State) {
-			m, err := st.AddMachine("series", JobHostUnits)
-			c.Assert(err, IsNil)
-			err = m.SetStatus(params.StatusStarted, "")
-			c.Assert(err, IsNil)
+		setUp: func(c *gc.C, st *State) {
+			m, err := st.AddMachine("quantal", JobHostUnits)
+			c.Assert(err, gc.IsNil)
+			err = m.SetStatus(params.StatusStarted, "", nil)
+			c.Assert(err, gc.IsNil)
 		},
 		change: watcher.Change{
 			C:  "statuses",
@@ -676,15 +729,16 @@ var allWatcherChangedTests = []struct {
 		},
 		expectContents: []params.EntityInfo{
 			&params.MachineInfo{
-				Id:     "0",
-				Status: params.StatusStarted,
+				Id:         "0",
+				Status:     params.StatusStarted,
+				StatusData: params.StatusData{},
 			},
 		},
 	},
 	// Service constraints changes
 	{
 		about: "no service in state -> do nothing",
-		setUp: func(c *C, st *State) {},
+		setUp: func(c *gc.C, st *State) {},
 		change: watcher.Change{
 			C:  "constraints",
 			Id: "s#wordpress",
@@ -695,7 +749,7 @@ var allWatcherChangedTests = []struct {
 			Name:        "wordpress",
 			Constraints: constraints.MustParse("mem=99M"),
 		}},
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "constraints",
 			Id: "s#wordpress",
@@ -710,11 +764,11 @@ var allWatcherChangedTests = []struct {
 			Name:        "wordpress",
 			Constraints: constraints.MustParse("mem=99M cpu-cores=2 cpu-power=4"),
 		}},
-		setUp: func(c *C, st *State) {
+		setUp: func(c *gc.C, st *State) {
 			svc, err := st.AddService("wordpress", AddTestingCharm(c, st, "wordpress"))
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			err = svc.SetConstraints(constraints.MustParse("mem=4G cpu-cores= arch=amd64"))
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 		},
 		change: watcher.Change{
 			C:  "constraints",
@@ -730,82 +784,109 @@ var allWatcherChangedTests = []struct {
 	// Service config changes.
 	{
 		about: "no service in state -> do nothing",
-		setUp: func(c *C, st *State) {},
+		setUp: func(c *gc.C, st *State) {},
 		change: watcher.Change{
 			C:  "settings",
-			Id: "s#wordpress#local:series/series-wordpress-3",
+			Id: "s#wordpress#local:quantal/quantal-wordpress-3",
 		},
 	}, {
 		about: "no change if service is not in backing",
 		add: []params.EntityInfo{&params.ServiceInfo{
 			Name:     "wordpress",
-			CharmURL: "local:series/series-wordpress-3",
+			CharmURL: "local:quantal/quantal-wordpress-3",
 		}},
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "settings",
-			Id: "s#wordpress#local:series/series-wordpress-3",
+			Id: "s#wordpress#local:quantal/quantal-wordpress-3",
 		},
 		expectContents: []params.EntityInfo{&params.ServiceInfo{
 			Name:     "wordpress",
-			CharmURL: "local:series/series-wordpress-3",
+			CharmURL: "local:quantal/quantal-wordpress-3",
 		}},
 	}, {
 		about: "service config is changed if service exists in the store with the same URL",
 		add: []params.EntityInfo{&params.ServiceInfo{
 			Name:     "wordpress",
-			CharmURL: "local:series/series-wordpress-3",
-			Config:   map[string]interface{}{"foo": "bar"},
+			CharmURL: "local:quantal/quantal-wordpress-3",
+			Config:   charm.Settings{"foo": "bar"},
 		}},
-		setUp: func(c *C, st *State) {
+		setUp: func(c *gc.C, st *State) {
 			svc, err := st.AddService("wordpress", AddTestingCharm(c, st, "wordpress"))
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			setServiceConfigAttr(c, svc, "blog-title", "foo")
 		},
 		change: watcher.Change{
 			C:  "settings",
-			Id: "s#wordpress#local:series/series-wordpress-3",
+			Id: "s#wordpress#local:quantal/quantal-wordpress-3",
 		},
 		expectContents: []params.EntityInfo{
 			&params.ServiceInfo{
 				Name:     "wordpress",
-				CharmURL: "local:series/series-wordpress-3",
-				Config:   map[string]interface{}{"blog-title": "foo"},
+				CharmURL: "local:quantal/quantal-wordpress-3",
+				Config:   charm.Settings{"blog-title": "foo"},
+			},
+		},
+	}, {
+		about: "service config is unescaped when reading from the backing store",
+		add: []params.EntityInfo{&params.ServiceInfo{
+			Name:     "wordpress",
+			CharmURL: "local:quantal/quantal-wordpress-3",
+			Config:   charm.Settings{"key.dotted": "bar"},
+		}},
+		setUp: func(c *gc.C, st *State) {
+			testCharm := AddCustomCharm(
+				c, st, "wordpress",
+				"config.yaml", dottedConfig,
+				"quantal", 3)
+			svc, err := st.AddService("wordpress", testCharm)
+			c.Assert(err, gc.IsNil)
+			setServiceConfigAttr(c, svc, "key.dotted", "foo")
+		},
+		change: watcher.Change{
+			C:  "settings",
+			Id: "s#wordpress#local:quantal/quantal-wordpress-3",
+		},
+		expectContents: []params.EntityInfo{
+			&params.ServiceInfo{
+				Name:     "wordpress",
+				CharmURL: "local:quantal/quantal-wordpress-3",
+				Config:   charm.Settings{"key.dotted": "foo"},
 			},
 		},
 	}, {
 		about: "service config is unchanged if service exists in the store with a different URL",
 		add: []params.EntityInfo{&params.ServiceInfo{
 			Name:     "wordpress",
-			CharmURL: "local:series/series-wordpress-2", // Note different revno.
-			Config:   map[string]interface{}{"foo": "bar"},
+			CharmURL: "local:quantal/quantal-wordpress-2", // Note different revno.
+			Config:   charm.Settings{"foo": "bar"},
 		}},
-		setUp: func(c *C, st *State) {
+		setUp: func(c *gc.C, st *State) {
 			svc, err := st.AddService("wordpress", AddTestingCharm(c, st, "wordpress"))
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 			setServiceConfigAttr(c, svc, "blog-title", "foo")
 		},
 		change: watcher.Change{
 			C:  "settings",
-			Id: "s#wordpress#local:series/series-wordpress-3",
+			Id: "s#wordpress#local:quantal/quantal-wordpress-3",
 		},
 		expectContents: []params.EntityInfo{
 			&params.ServiceInfo{
 				Name:     "wordpress",
-				CharmURL: "local:series/series-wordpress-2",
-				Config:   map[string]interface{}{"foo": "bar"},
+				CharmURL: "local:quantal/quantal-wordpress-2",
+				Config:   charm.Settings{"foo": "bar"},
 			},
 		},
 	}, {
 		about: "non-service config change is ignored",
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "settings",
 			Id: "m#0",
 		},
 	}, {
 		about: "service config change with no charm url is ignored",
-		setUp: func(*C, *State) {},
+		setUp: func(*gc.C, *State) {},
 		change: watcher.Change{
 			C:  "settings",
 			Id: "s#foo",
@@ -813,15 +894,12 @@ var allWatcherChangedTests = []struct {
 	},
 }
 
-func setServiceConfigAttr(c *C, svc *Service, attr string, val interface{}) {
-	cfg, err := svc.Config()
-	c.Assert(err, IsNil)
-	cfg.Set("blog-title", val)
-	_, err = cfg.Write()
-	c.Assert(err, IsNil)
+func setServiceConfigAttr(c *gc.C, svc *Service, attr string, val interface{}) {
+	err := svc.UpdateConfigSettings(charm.Settings{attr: val})
+	c.Assert(err, gc.IsNil)
 }
 
-func (s *storeManagerStateSuite) TestChanged(c *C) {
+func (s *storeManagerStateSuite) TestChanged(c *gc.C) {
 	collections := map[string]*mgo.Collection{
 		"machines":    s.State.machines,
 		"units":       s.State.units,
@@ -844,7 +922,7 @@ func (s *storeManagerStateSuite) TestChanged(c *C) {
 		ch := test.change
 		ch.C = collections[ch.C].Name
 		err := b.Changed(all, test.change)
-		c.Assert(err, IsNil)
+		c.Assert(err, gc.IsNil)
 		assertEntitiesEqual(c, all.All(), test.expectContents)
 		s.Reset(c)
 	}
@@ -853,14 +931,14 @@ func (s *storeManagerStateSuite) TestChanged(c *C) {
 // StateWatcher tests the integration of the state watcher
 // with the state-based backing. Most of the logic is tested elsewhere -
 // this just tests end-to-end.
-func (s *storeManagerStateSuite) TestStateWatcher(c *C) {
-	m0, err := s.State.AddMachine("series", JobManageEnviron)
-	c.Assert(err, IsNil)
-	c.Assert(m0.Id(), Equals, "0")
+func (s *storeManagerStateSuite) TestStateWatcher(c *gc.C) {
+	m0, err := s.State.AddMachine("quantal", JobManageEnviron)
+	c.Assert(err, gc.IsNil)
+	c.Assert(m0.Id(), gc.Equals, "0")
 
-	m1, err := s.State.AddMachine("series", JobHostUnits)
-	c.Assert(err, IsNil)
-	c.Assert(m1.Id(), Equals, "1")
+	m1, err := s.State.AddMachine("quantal", JobHostUnits)
+	c.Assert(err, gc.IsNil)
+	c.Assert(m1.Id(), gc.Equals, "1")
 
 	b := newAllWatcherStateBacking(s.State)
 	aw := multiwatcher.NewStoreManager(b)
@@ -880,17 +958,17 @@ func (s *storeManagerStateSuite) TestStateWatcher(c *C) {
 	}}, "")
 
 	// Make some changes to the state.
-	err = m0.SetProvisioned("i-0", "bootstrap_nonce")
-	c.Assert(err, IsNil)
+	err = m0.SetProvisioned("i-0", "bootstrap_nonce", nil)
+	c.Assert(err, gc.IsNil)
 	err = m1.Destroy()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	err = m1.EnsureDead()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	err = m1.Remove()
-	c.Assert(err, IsNil)
-	m2, err := s.State.AddMachine("series", JobManageEnviron)
-	c.Assert(err, IsNil)
-	c.Assert(m2.Id(), Equals, "2")
+	c.Assert(err, gc.IsNil)
+	m2, err := s.State.AddMachine("quantal", JobManageEnviron)
+	c.Assert(err, gc.IsNil)
+	c.Assert(m2.Id(), gc.Equals, "2")
 	s.State.StartSync()
 
 	// Check that we see the changes happen within a
@@ -901,7 +979,7 @@ func (s *storeManagerStateSuite) TestStateWatcher(c *C) {
 		if err == errTimeout {
 			break
 		}
-		c.Assert(err, IsNil)
+		c.Assert(err, gc.IsNil)
 		deltas = append(deltas, d...)
 	}
 	checkDeltasEqual(c, b, deltas, []params.Delta{{
@@ -924,10 +1002,10 @@ func (s *storeManagerStateSuite) TestStateWatcher(c *C) {
 	}})
 
 	err = w.Stop()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 
 	_, err = w.Next()
-	c.Assert(err, ErrorMatches, "state watcher was stopped")
+	c.Assert(err, gc.ErrorMatches, "state watcher was stopped")
 }
 
 type entityInfoSlice []params.EntityInfo
@@ -949,7 +1027,7 @@ func (s entityInfoSlice) Less(i, j int) bool {
 
 var errTimeout = errors.New("no change received in sufficient time")
 
-func getNext(c *C, w *multiwatcher.Watcher, timeout time.Duration) ([]params.Delta, error) {
+func getNext(c *gc.C, w *multiwatcher.Watcher, timeout time.Duration) ([]params.Delta, error) {
 	var deltas []params.Delta
 	var err error
 	ch := make(chan struct{}, 1)
@@ -965,10 +1043,10 @@ func getNext(c *C, w *multiwatcher.Watcher, timeout time.Duration) ([]params.Del
 	return nil, errTimeout
 }
 
-func checkNext(c *C, w *multiwatcher.Watcher, b multiwatcher.Backing, deltas []params.Delta, expectErr string) {
+func checkNext(c *gc.C, w *multiwatcher.Watcher, b multiwatcher.Backing, deltas []params.Delta, expectErr string) {
 	d, err := getNext(c, w, 1*time.Second)
 	if expectErr != "" {
-		c.Check(err, ErrorMatches, expectErr)
+		c.Check(err, gc.ErrorMatches, expectErr)
 		return
 	}
 	checkDeltasEqual(c, b, d, deltas)
@@ -976,8 +1054,8 @@ func checkNext(c *C, w *multiwatcher.Watcher, b multiwatcher.Backing, deltas []p
 
 // deltas are returns in arbitrary order, so we compare
 // them as sets.
-func checkDeltasEqual(c *C, b multiwatcher.Backing, d0, d1 []params.Delta) {
-	c.Check(deltaMap(d0, b), DeepEquals, deltaMap(d1, b))
+func checkDeltasEqual(c *gc.C, b multiwatcher.Backing, d0, d1 []params.Delta) {
+	c.Check(deltaMap(d0, b), gc.DeepEquals, deltaMap(d1, b))
 }
 
 func deltaMap(deltas []params.Delta, b multiwatcher.Backing) map[multiwatcher.InfoId]params.EntityInfo {
