@@ -1,3 +1,6 @@
+// Copyright 2012, 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 // The state package enables reading, observing, and changing
 // the state stored in MongoDB of a whole environment
 // managed by juju.
@@ -5,138 +8,106 @@ package state
 
 import (
 	"fmt"
+	"net/url"
+	"sort"
+	"strconv"
+	"strings"
+	"sync"
+
 	"labix.org/v2/mgo"
 	"labix.org/v2/mgo/bson"
 	"labix.org/v2/mgo/txn"
+	"launchpad.net/loggo"
+
 	"launchpad.net/juju-core/charm"
 	"launchpad.net/juju-core/constraints"
 	"launchpad.net/juju-core/environs/config"
-	"launchpad.net/juju-core/log"
+	"launchpad.net/juju-core/errors"
+	"launchpad.net/juju-core/instance"
+	"launchpad.net/juju-core/names"
 	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/multiwatcher"
 	"launchpad.net/juju-core/state/presence"
 	"launchpad.net/juju-core/state/watcher"
 	"launchpad.net/juju-core/utils"
-	"launchpad.net/juju-core/version"
-	"net/url"
-	"regexp"
-	"sort"
-	"strconv"
-	"strings"
-	"sync"
 )
 
+var logger = loggo.GetLogger("juju.state")
+
 // TODO(niemeyer): This must not be exported.
 type D []bson.DocElem
 
-// Tools describes a particular set of juju tools and where to find them.
-type Tools struct {
-	version.Binary
-	URL string
-}
-
-type toolsDoc struct {
-	Version version.Binary
-	URL     string
-}
-
-func (t *Tools) GetBSON() (interface{}, error) {
-	if t == nil {
-		return nil, nil
-	}
-	return &toolsDoc{t.Binary, t.URL}, nil
-}
-
-func (t *Tools) SetBSON(raw bson.Raw) error {
-	if raw.Kind == 10 {
-		// Preserve the nil value in that case.
-		return bson.SetZero
-	}
-	var doc toolsDoc
-	if err := raw.Unmarshal(&doc); err != nil {
-		return err
-	}
-	t.Binary = doc.Version
-	t.URL = doc.URL
-	return nil
-}
-
-const serviceSnippet = "[a-z][a-z0-9]*(-[a-z0-9]*[a-z][a-z0-9]*)*"
-
-var (
-	validService = regexp.MustCompile("^" + serviceSnippet + "$")
-	validUnit    = regexp.MustCompile("^" + serviceSnippet + "(-[a-z0-9]*[a-z][a-z0-9]*)*/[0-9]+$")
-	validMachine = regexp.MustCompile("^0$|^[1-9][0-9]*$")
-)
-
 // BootstrapNonce is used as a nonce for the state server machine.
 const BootstrapNonce = "user-admin:bootstrap"
 
-// IsServiceName returns whether name is a valid service name.
-func IsServiceName(name string) bool {
-	return validService.MatchString(name)
-}
-
-// IsUnitName returns whether name is a valid unit name.
-func IsUnitName(name string) bool {
-	return validUnit.MatchString(name)
-}
-
-// IsMachineId returns whether id is a valid machine id.
-func IsMachineId(name string) bool {
-	return validMachine.MatchString(name)
-}
-
-// notFoundError represents the error that something is not found.
-type notFoundError struct {
-	msg string
-}
-
-func (e *notFoundError) Error() string {
-	return e.msg
-}
-
-// NotFoundf returns a error for which IsNotFound returns
-// true. The message for the error is made up from the given
-// arguments formatted as with fmt.Sprintf, with the
-// string " not found" appended.
-func NotFoundf(format string, args ...interface{}) error {
-	return &notFoundError{fmt.Sprintf(format+" not found", args...)}
-}
-
-func IsNotFound(err error) bool {
-	_, ok := err.(*notFoundError)
-	return ok
-}
-
 // State represents the state of an environment
 // managed by juju.
 type State struct {
-	info           *Info
-	db             *mgo.Database
-	environments   *mgo.Collection
-	charms         *mgo.Collection
-	machines       *mgo.Collection
-	relations      *mgo.Collection
-	relationScopes *mgo.Collection
-	services       *mgo.Collection
-	settings       *mgo.Collection
-	settingsrefs   *mgo.Collection
-	constraints    *mgo.Collection
-	units          *mgo.Collection
-	users          *mgo.Collection
-	presence       *mgo.Collection
-	cleanups       *mgo.Collection
-	annotations    *mgo.Collection
-	statuses       *mgo.Collection
-	runner         *txn.Runner
-	watcher        *watcher.Watcher
-	pwatcher       *presence.Watcher
+	info             *Info
+	db               *mgo.Database
+	environments     *mgo.Collection
+	charms           *mgo.Collection
+	machines         *mgo.Collection
+	instanceData     *mgo.Collection
+	containerRefs    *mgo.Collection
+	relations        *mgo.Collection
+	relationScopes   *mgo.Collection
+	services         *mgo.Collection
+	minUnits         *mgo.Collection
+	settings         *mgo.Collection
+	settingsrefs     *mgo.Collection
+	constraints      *mgo.Collection
+	units            *mgo.Collection
+	users            *mgo.Collection
+	presence         *mgo.Collection
+	cleanups         *mgo.Collection
+	annotations      *mgo.Collection
+	statuses         *mgo.Collection
+	runner           *txn.Runner
+	transactionHooks chan ([]transactionHook)
+	watcher          *watcher.Watcher
+	pwatcher         *presence.Watcher
 	// mu guards allManager.
 	mu         sync.Mutex
 	allManager *multiwatcher.StoreManager
 }
 
+// transactionHook holds a pair of functions to be called before and after a
+// mgo/txn transaction is run. It is only used in testing.
+type transactionHook struct {
+	Before func()
+	After  func()
+}
+
+// runTransaction runs the supplied operations as a single mgo/txn transaction,
+// and includes a mechanism whereby tests can use SetTransactionHooks to induce
+// arbitrary state mutations before and after particular transactions.
+func (st *State) runTransaction(ops []txn.Op) error {
+	transactionHooks := <-st.transactionHooks
+	st.transactionHooks <- nil
+	if len(transactionHooks) > 0 {
+		defer func() {
+			if transactionHooks[0].After != nil {
+				transactionHooks[0].After()
+			}
+			if <-st.transactionHooks != nil {
+				panic("concurrent use of transaction hooks")
+			}
+			st.transactionHooks <- transactionHooks[1:]
+		}()
+		if transactionHooks[0].Before != nil {
+			transactionHooks[0].Before()
+		}
+	}
+	return st.runner.Run(ops, "", nil)
+}
+
+// Ping probes the state's database connection to ensure
+// that it is still alive.
+func (st *State) Ping() error {
+	return st.db.Session.Ping()
+}
+
 func (st *State) Watch() *multiwatcher.Watcher {
 	st.mu.Lock()
 	if st.allManager == nil {
@@ -152,7 +123,7 @@ func (st *State) EnvironConfig() (*config.Config, error) {
 		return nil, err
 	}
 	attrs := settings.Map()
-	return config.New(attrs)
+	return config.New(config.NoDefaults, attrs)
 }
 
 // checkEnvironConfig returns an error if the config is definitely invalid.
@@ -198,26 +169,58 @@ func (st *State) SetEnvironConstraints(cons constraints.Value) error {
 // supplied series. The machine's constraints will be taken from the
 // environment constraints.
 func (st *State) AddMachine(series string, jobs ...MachineJob) (m *Machine, err error) {
-	return st.addMachine(series, "", "", jobs)
+	return st.addMachine(&AddMachineParams{Series: series, Jobs: jobs})
+}
+
+// AddMachineWithConstraints adds a new machine configured to run the supplied jobs on the
+// supplied series. The machine's constraints and other configuration will be taken from
+// the supplied params struct.
+func (st *State) AddMachineWithConstraints(params *AddMachineParams) (m *Machine, err error) {
+	if params.InstanceId != "" {
+		return nil, fmt.Errorf("cannot specify an instance id when adding a new machine")
+	}
+	if params.Nonce != "" {
+		return nil, fmt.Errorf("cannot specify a nonce when adding a new machine")
+	}
+
+	// TODO(wallyworld) - if a container is required, and when the actual machine characteristics
+	// are made available, we need to check the machine constraints to ensure the container can be
+	// created on the specifed machine.
+	// ie it makes no sense asking for a 16G container on a machine with 8G.
+
+	return st.addMachine(params)
 }
 
 // InjectMachine adds a new machine, corresponding to an existing provider
-// instance, configured to run the supplied jobs on the supplied series. The
-// machine's constraints will be taken from the environment constraints.
-func (st *State) InjectMachine(series string, instanceId InstanceId, jobs ...MachineJob) (m *Machine, err error) {
-	if instanceId == "" {
+// instance, configured according to the supplied params struct.
+func (st *State) InjectMachine(params *AddMachineParams) (m *Machine, err error) {
+	if params.InstanceId == "" {
 		return nil, fmt.Errorf("cannot inject a machine without an instance id")
 	}
-	return st.addMachine(series, instanceId, BootstrapNonce, jobs)
+	if params.Nonce == "" {
+		return nil, fmt.Errorf("cannot inject a machine without a nonce")
+	}
+	return st.addMachine(params)
 }
 
-func (st *State) addMachineOps(mdoc *machineDoc, cons constraints.Value) (*machineDoc, []txn.Op, error) {
+// containerRefParams specify how a machineContainers document is to be created.
+type containerRefParams struct {
+	hostId      string
+	newHost     bool
+	hostOnly    bool
+	containerId string
+}
+
+func (st *State) addMachineOps(mdoc *machineDoc, metadata *instanceData, cons constraints.Value, containerParams *containerRefParams) (*machineDoc, []txn.Op, error) {
 	if mdoc.Series == "" {
 		return nil, nil, fmt.Errorf("no series specified")
 	}
 	if len(mdoc.Jobs) == 0 {
 		return nil, nil, fmt.Errorf("no jobs specified")
 	}
+	if containerParams.hostId != "" && mdoc.ContainerType == "" {
+		return nil, nil, fmt.Errorf("no container type specified")
+	}
 	jset := make(map[MachineJob]bool)
 	for _, j := range mdoc.Jobs {
 		if jset[j] {
@@ -225,15 +228,34 @@ func (st *State) addMachineOps(mdoc *machineDoc, cons constraints.Value) (*machi
 		}
 		jset[j] = true
 	}
-	seq, err := st.sequence("machine")
-	if err != nil {
-		return nil, nil, err
+	if containerParams.hostId == "" {
+		// we are creating a new machine instance (not a container).
+		seq, err := st.sequence("machine")
+		if err != nil {
+			return nil, nil, err
+		}
+		mdoc.Id = strconv.Itoa(seq)
+		containerParams.hostId = mdoc.Id
+		containerParams.newHost = true
+	}
+	if mdoc.ContainerType != "" {
+		// we are creating a container so set up a namespaced id.
+		seq, err := st.sequence(fmt.Sprintf("machine%s%sContainer", containerParams.hostId, mdoc.ContainerType))
+		if err != nil {
+			return nil, nil, err
+		}
+		mdoc.Id = fmt.Sprintf("%s/%s/%d", containerParams.hostId, mdoc.ContainerType, seq)
+		containerParams.containerId = mdoc.Id
 	}
-	mdoc.Id = strconv.Itoa(seq)
 	mdoc.Life = Alive
 	sdoc := statusDoc{
 		Status: params.StatusPending,
 	}
+	// Machine constraints do not use a container constraint value.
+	// Both provisioning and deployment constraints use the same constraints.Value struct
+	// so here we clear the container value. Provisioning ignores the container value but
+	// clearing it avoids potential confusion.
+	cons.Container = nil
 	ops := []txn.Op{
 		{
 			C:      st.machines.Name,
@@ -244,29 +266,114 @@ func (st *State) addMachineOps(mdoc *machineDoc, cons constraints.Value) (*machi
 		createConstraintsOp(st, machineGlobalKey(mdoc.Id), cons),
 		createStatusOp(st, machineGlobalKey(mdoc.Id), sdoc),
 	}
+	if metadata != nil {
+		ops = append(ops, txn.Op{
+			C:      st.instanceData.Name,
+			Id:     mdoc.Id,
+			Assert: txn.DocMissing,
+			Insert: *metadata,
+		})
+	}
+	ops = append(ops, createContainerRefOp(st, containerParams)...)
 	return mdoc, ops, nil
 }
 
+// AddMachineParams encapsulates the parameters used to create a new machine.
+type AddMachineParams struct {
+	Series                  string
+	Constraints             constraints.Value
+	ParentId                string
+	ContainerType           instance.ContainerType
+	InstanceId              instance.Id
+	HardwareCharacteristics instance.HardwareCharacteristics
+	Nonce                   string
+	Jobs                    []MachineJob
+}
+
+// addMachineContainerOps returns txn operations and associated Mongo records used to create a new machine,
+// accounting for the fact that a machine may require a container and may require instance data.
+// This method exists to cater for:
+// 1. InjectMachine, which is used to record in state an instantiated bootstrap node. When adding
+// a machine to state so that it is provisioned normally, the instance id is not known at this point.
+// 2. AssignToNewMachine, which is used to create a new machine on which to deploy a unit.
+func (st *State) addMachineContainerOps(params *AddMachineParams, cons constraints.Value) ([]txn.Op, *instanceData, *containerRefParams, error) {
+	var instData *instanceData
+	if params.InstanceId != "" {
+		instData = &instanceData{
+			InstanceId: params.InstanceId,
+			Arch:       params.HardwareCharacteristics.Arch,
+			Mem:        params.HardwareCharacteristics.Mem,
+			RootDisk:   params.HardwareCharacteristics.RootDisk,
+			CpuCores:   params.HardwareCharacteristics.CpuCores,
+			CpuPower:   params.HardwareCharacteristics.CpuPower,
+			Tags:       params.HardwareCharacteristics.Tags,
+		}
+	}
+	var ops []txn.Op
+	var containerParams = &containerRefParams{hostId: params.ParentId, hostOnly: true}
+	// If we are creating a container, first create the host (parent) machine if necessary.
+	if params.ContainerType != "" {
+		containerParams.hostOnly = false
+		if params.ParentId == "" {
+			// No parent machine is specified so create one.
+			mdoc := &machineDoc{
+				Series: params.Series,
+				Jobs:   params.Jobs,
+				Clean:  true,
+			}
+			mdoc, parentOps, err := st.addMachineOps(mdoc, instData, cons, &containerRefParams{})
+			if err != nil {
+				return nil, nil, nil, err
+			}
+			ops = parentOps
+			containerParams.hostId = mdoc.Id
+			containerParams.newHost = true
+		} else {
+			// If a parent machine is specified, make sure it exists.
+			_, err := st.Machine(containerParams.hostId)
+			if err != nil {
+				return nil, nil, nil, err
+			}
+		}
+	}
+	return ops, instData, containerParams, nil
+}
+
 // addMachine implements AddMachine and InjectMachine.
-func (st *State) addMachine(series string, instanceId InstanceId, nonce string, jobs []MachineJob) (m *Machine, err error) {
-	defer utils.ErrorContextf(&err, "cannot add a new machine")
+func (st *State) addMachine(params *AddMachineParams) (m *Machine, err error) {
+	msg := "cannot add a new machine"
+	if params.ParentId != "" || params.ContainerType != "" {
+		msg = "cannot add a new container"
+	}
+	defer utils.ErrorContextf(&err, msg)
 
 	cons, err := st.EnvironConstraints()
 	if err != nil {
 		return nil, err
 	}
-	mdoc := &machineDoc{
-		Series:     series,
-		InstanceId: instanceId,
-		Nonce:      nonce,
-		Jobs:       jobs,
-	}
-	mdoc, ops, err := st.addMachineOps(mdoc, cons)
+	cons = params.Constraints.WithFallbacks(cons)
+
+	ops, instData, containerParams, err := st.addMachineContainerOps(params, cons)
 	if err != nil {
 		return nil, err
 	}
+	mdoc := &machineDoc{
+		Series:        params.Series,
+		ContainerType: string(params.ContainerType),
+		Jobs:          params.Jobs,
+		Clean:         true,
+	}
+	if mdoc.ContainerType == "" {
+		mdoc.InstanceId = params.InstanceId
+		mdoc.Nonce = params.Nonce
+	}
+	mdoc, machineOps, err := st.addMachineOps(mdoc, instData, cons, containerParams)
+	if err != nil {
+		return nil, err
+	}
+	ops = append(ops, machineOps...)
 
-	err = st.runner.Run(ops, "", nil)
+	err = st.runTransaction(ops)
 	if err != nil {
 		return nil, err
 	}
@@ -308,10 +415,49 @@ type machineDocSlice []machineDoc
 func (ms machineDocSlice) Len() int      { return len(ms) }
 func (ms machineDocSlice) Swap(i, j int) { ms[i], ms[j] = ms[j], ms[i] }
 func (ms machineDocSlice) Less(i, j int) bool {
-	// There's nothing we can do with errors at this point.
-	m1, _ := strconv.Atoi(ms[i].Id)
-	m2, _ := strconv.Atoi(ms[j].Id)
-	return m1 < m2
+	return machineIdLessThan(ms[i].Id, ms[j].Id)
+}
+
+// machineIdLessThan returns true if id1 < id2, false otherwise.
+// Machine ids may include "/" separators if they are for a container so
+// the comparison is done by comparing the id component values from
+// left to right (most significant part to least significant). Ids for
+// host machines are always less than ids for their containers.
+func machineIdLessThan(id1, id2 string) bool {
+	// Most times, we are dealing with host machines and not containers, so we will
+	// try interpreting the ids as ints - this will be faster than dealing with the
+	// container ids below.
+	mint1, err1 := strconv.Atoi(id1)
+	mint2, err2 := strconv.Atoi(id2)
+	if err1 == nil && err2 == nil {
+		return mint1 < mint2
+	}
+	// We have at least one container id so it gets complicated.
+	idParts1 := strings.Split(id1, "/")
+	idParts2 := strings.Split(id2, "/")
+	nrParts1 := len(idParts1)
+	nrParts2 := len(idParts2)
+	minLen := nrParts1
+	if nrParts2 < minLen {
+		minLen = nrParts2
+	}
+	for x := 0; x < minLen; x++ {
+		m1 := idParts1[x]
+		m2 := idParts2[x]
+		if m1 == m2 {
+			continue
+		}
+		// See if the id part is a container type, and if so compare directly.
+		if x%2 == 1 {
+			return m1 < m2
+		}
+		// Compare the integer ids.
+		// There's nothing we can do with errors at this point.
+		mint1, _ := strconv.Atoi(m1)
+		mint2, _ := strconv.Atoi(m2)
+		return mint1 < mint2
+	}
+	return nrParts1 < nrParts2
 }
 
 // Machine returns the machine with the given id.
@@ -320,7 +466,7 @@ func (st *State) Machine(id string) (*Machine, error) {
 	sel := D{{"_id", id}}
 	err := st.machines.Find(sel).One(mdoc)
 	if err == mgo.ErrNotFound {
-		return nil, NotFoundf("machine %s", id)
+		return nil, errors.NotFoundf("machine %s", id)
 	}
 	if err != nil {
 		return nil, fmt.Errorf("cannot get machine %s: %v", id, err)
@@ -328,93 +474,23 @@ func (st *State) Machine(id string) (*Machine, error) {
 	return newMachine(st, mdoc), nil
 }
 
-// Tagger represents entities with a tag.
-type Tagger interface {
-	Tag() string
-}
-
-// Authenticator represents entites capable of handling password
-// authentication.
-type Authenticator interface {
-	Refresh() error
-	SetPassword(pass string) error
-	PasswordValid(pass string) bool
-}
-
-// TaggedAuthenticator represents tagged entities capable of authentication.
-type TaggedAuthenticator interface {
-	Authenticator
-	Tagger
-}
-
-// Annotator represents entities capable of handling annotations.
-type Annotator interface {
-	Annotation(key string) (string, error)
-	Annotations() (map[string]string, error)
-	SetAnnotations(pairs map[string]string) error
-}
-
-// TaggedAnnotator represents tagged entities capable of handling annotations.
-type TaggedAnnotator interface {
-	Annotator
-	Tagger
-}
-
-// Authenticator attempts to return a TaggedAuthenticator with the given name.
-func (st *State) Authenticator(name string) (TaggedAuthenticator, error) {
-	e, err := st.entity(name)
-	if err != nil {
-		return nil, err
-	}
-	if e, ok := e.(TaggedAuthenticator); ok {
-		return e, nil
-	}
-	return nil, fmt.Errorf("entity %q does not support authentication", name)
-}
-
-// Annotator attempts to return aa TaggedAnnotator with the given name.
-func (st *State) Annotator(name string) (TaggedAnnotator, error) {
-	e, err := st.entity(name)
-	if err != nil {
-		return nil, err
-	}
-	if e, ok := e.(TaggedAnnotator); ok {
-		return e, nil
-	}
-	return nil, fmt.Errorf("entity %q does not support annotations", name)
-}
-
-// entity returns the entity for the given tag.
-func (st *State) entity(tag string) (interface{}, error) {
-	i := strings.Index(tag, "-")
-	if i <= 0 || i >= len(tag)-1 {
-		return nil, fmt.Errorf("invalid entity tag %q", tag)
-	}
-	prefix, id := tag[0:i], tag[i+1:]
-	switch prefix {
-	case "machine":
-		if !IsMachineId(id) {
-			return nil, fmt.Errorf("invalid entity tag %q", tag)
-		}
+// FindEntity returns the entity with the given tag.
+//
+// The returned value can be of type *Machine, *Unit,
+// *User, *Service or *Environment, depending
+// on the tag.
+func (st *State) FindEntity(tag string) (Entity, error) {
+	kind, id, err := names.ParseTag(tag, "")
+	switch kind {
+	case names.MachineTagKind:
 		return st.Machine(id)
-	case "unit":
-		i := strings.LastIndex(id, "-")
-		if i == -1 {
-			return nil, fmt.Errorf("invalid entity tag %q", tag)
-		}
-		name := id[:i] + "/" + id[i+1:]
-		if !IsUnitName(name) {
-			return nil, fmt.Errorf("invalid entity tag %q", tag)
-		}
-		return st.Unit(name)
-	case "user":
+	case names.UnitTagKind:
+		return st.Unit(id)
+	case names.UserTagKind:
 		return st.User(id)
-	case "service":
-		if !IsServiceName(id) {
-			return nil, fmt.Errorf("invalid entity tag %q", tag)
-		}
+	case names.ServiceTagKind:
 		return st.Service(id)
-	case "environment":
+	case names.EnvironTagKind:
 		conf, err := st.EnvironConfig()
 		if err != nil {
 			return nil, err
@@ -422,40 +498,35 @@ func (st *State) entity(tag string) (interface{}, error) {
 		// Return an invalid entity error if the requested environment is not
 		// the current one.
 		if id != conf.Name() {
-			return nil, fmt.Errorf("invalid entity tag %q", tag)
+			return nil, errors.NotFoundf("environment %q", id)
 		}
 		return st.Environment()
+	case names.RelationTagKind:
+		return st.KeyRelation(id)
 	}
-	return nil, fmt.Errorf("invalid entity tag %q", tag)
+	return nil, err
 }
 
-// ParseTag, given an entity tag, returns the collection name and id
+// parseTag, given an entity tag, returns the collection name and id
 // of the entity document.
-func (st *State) ParseTag(tag string) (string, string, error) {
-	parts := strings.SplitN(tag, "-", 2)
-	if len(parts) != 2 {
-		return "", "", fmt.Errorf("invalid entity name %q", tag)
+func (st *State) parseTag(tag string) (coll string, id string, err error) {
+	kind, id, err := names.ParseTag(tag, "")
+	if err != nil {
+		return "", "", err
 	}
-	id := parts[1]
-	var coll string
-	switch parts[0] {
-	case "machine":
+	switch kind {
+	case names.MachineTagKind:
 		coll = st.machines.Name
-	case "service":
+	case names.ServiceTagKind:
 		coll = st.services.Name
-	case "unit":
+	case names.UnitTagKind:
 		coll = st.units.Name
-		// Handle replacements occurring when an entity name is created
-		// for a unit.
-		idx := strings.LastIndex(id, "-")
-		if idx == -1 {
-			return "", "", fmt.Errorf("invalid entity name %q", tag)
-		}
-		id = id[:idx] + "/" + id[idx+1:]
-	case "user":
+	case names.UserTagKind:
 		coll = st.users.Name
+	case names.RelationTagKind:
+		coll = st.relations.Name
 	default:
-		return "", "", fmt.Errorf("invalid entity name %q", tag)
+		return "", "", fmt.Errorf("%q is not a valid collection tag", tag)
 	}
 	return coll, id, nil
 }
@@ -483,7 +554,7 @@ func (st *State) Charm(curl *charm.URL) (*Charm, error) {
 	cdoc := &charmDoc{}
 	err := st.charms.Find(D{{"_id", curl}}).One(cdoc)
 	if err == mgo.ErrNotFound {
-		return nil, NotFoundf("charm %q", curl)
+		return nil, errors.NotFoundf("charm %q", curl)
 	}
 	if err != nil {
 		return nil, fmt.Errorf("cannot get charm %q: %v", curl, err)
@@ -530,7 +601,7 @@ func (st *State) addPeerRelationsOps(serviceName string, peers map[string]charm.
 func (st *State) AddService(name string, ch *Charm) (service *Service, err error) {
 	defer utils.ErrorContextf(&err, "cannot add service %q", name)
 	// Sanity checks.
-	if !IsServiceName(name) {
+	if !names.IsService(name) {
 		return nil, fmt.Errorf("invalid name")
 	}
 	if ch == nil {
@@ -576,7 +647,7 @@ func (st *State) AddService(name string, ch *Charm) (service *Service, err error
 	// Run the transaction; happily, there's never any reason to retry,
 	// because all the possible failed assertions imply that the service
 	// already exists.
-	if err := st.runner.Run(ops, "", nil); err == txn.ErrAborted {
+	if err := st.runTransaction(ops); err == txn.ErrAborted {
 		return nil, fmt.Errorf("service already exists")
 	} else if err != nil {
 		return nil, err
@@ -590,14 +661,14 @@ func (st *State) AddService(name string, ch *Charm) (service *Service, err error
 
 // Service returns a service state by name.
 func (st *State) Service(name string) (service *Service, err error) {
-	if !IsServiceName(name) {
+	if !names.IsService(name) {
 		return nil, fmt.Errorf("%q is not a valid service name", name)
 	}
 	sdoc := &serviceDoc{}
 	sel := D{{"_id", name}}
 	err = st.services.Find(sel).One(sdoc)
 	if err == mgo.ErrNotFound {
-		return nil, NotFoundf("service %q", name)
+		return nil, errors.NotFoundf("service %q", name)
 	}
 	if err != nil {
 		return nil, fmt.Errorf("cannot get service %q: %v", name, err)
@@ -772,7 +843,7 @@ func (st *State) AddRelation(eps ...Endpoint) (r *Relation, err error) {
 		series := map[string]bool{}
 		for _, ep := range eps {
 			svc, err := st.Service(ep.ServiceName)
-			if IsNotFound(err) {
+			if errors.IsNotFoundError(err) {
 				return nil, fmt.Errorf("service %q does not exist", ep.ServiceName)
 			} else if err != nil {
 				return nil, err
@@ -818,7 +889,7 @@ func (st *State) AddRelation(eps ...Endpoint) (r *Relation, err error) {
 			Insert: doc,
 		})
 		// Run the transaction, and retry on abort.
-		if err = st.runner.Run(ops, "", nil); err == txn.ErrAborted {
+		if err = st.runTransaction(ops); err == txn.ErrAborted {
 			continue
 		} else if err != nil {
 			return nil, err
@@ -830,11 +901,16 @@ func (st *State) AddRelation(eps ...Endpoint) (r *Relation, err error) {
 
 // EndpointsRelation returns the existing relation with the given endpoints.
 func (st *State) EndpointsRelation(endpoints ...Endpoint) (*Relation, error) {
+	return st.KeyRelation(relationKey(endpoints))
+}
+
+// KeyRelation returns the existing relation with the given key (which can
+// be derived unambiguously from the relation's endpoints).
+func (st *State) KeyRelation(key string) (*Relation, error) {
 	doc := relationDoc{}
-	key := relationKey(endpoints)
 	err := st.relations.Find(D{{"_id", key}}).One(&doc)
 	if err == mgo.ErrNotFound {
-		return nil, NotFoundf("relation %q", key)
+		return nil, errors.NotFoundf("relation %q", key)
 	}
 	if err != nil {
 		return nil, fmt.Errorf("cannot get relation %q: %v", key, err)
@@ -847,7 +923,7 @@ func (st *State) Relation(id int) (*Relation, error) {
 	doc := relationDoc{}
 	err := st.relations.Find(D{{"id", id}}).One(&doc)
 	if err == mgo.ErrNotFound {
-		return nil, NotFoundf("relation %d", id)
+		return nil, errors.NotFoundf("relation %d", id)
 	}
 	if err != nil {
 		return nil, fmt.Errorf("cannot get relation %d: %v", id, err)
@@ -857,13 +933,13 @@ func (st *State) Relation(id int) (*Relation, error) {
 
 // Unit returns a unit by name.
 func (st *State) Unit(name string) (*Unit, error) {
-	if !IsUnitName(name) {
+	if !names.IsUnit(name) {
 		return nil, fmt.Errorf("%q is not a valid unit name", name)
 	}
 	doc := unitDoc{}
 	err := st.units.FindId(name).One(&doc)
 	if err == mgo.ErrNotFound {
-		return nil, NotFoundf("unit %q", name)
+		return nil, errors.NotFoundf("unit %q", name)
 	}
 	if err != nil {
 		return nil, fmt.Errorf("cannot get unit %q: %v", name, err)
@@ -878,7 +954,7 @@ func (st *State) DestroyUnits(names ...string) (err error) {
 	for _, name := range names {
 		unit, err := st.Unit(name)
 		switch {
-		case IsNotFound(err):
+		case errors.IsNotFoundError(err):
 			err = fmt.Errorf("unit %q does not exist", name)
 		case err != nil:
 		case unit.Life() != Alive:
@@ -901,7 +977,7 @@ func (st *State) DestroyMachines(ids ...string) (err error) {
 	for _, id := range ids {
 		machine, err := st.Machine(id)
 		switch {
-		case IsNotFound(err):
+		case errors.IsNotFoundError(err):
 			err = fmt.Errorf("machine %s does not exist", id)
 		case err != nil:
 		case machine.Life() != Alive:
@@ -944,28 +1020,26 @@ func (st *State) AssignUnit(u *Unit, policy AssignmentPolicy) (err error) {
 			return err
 		}
 		return u.AssignToMachine(m)
-	case AssignUnused:
-		if _, err = u.AssignToUnusedMachine(); err != noUnusedMachines {
+	case AssignClean:
+		if _, err = u.AssignToCleanMachine(); err != noCleanMachines {
 			return err
 		}
-		return u.AssignToNewMachine()
+		return u.AssignToNewMachineOrContainer()
+	case AssignCleanEmpty:
+		if _, err = u.AssignToCleanEmptyMachine(); err != noCleanMachines {
+			return err
+		}
+		return u.AssignToNewMachineOrContainer()
 	case AssignNew:
 		return u.AssignToNewMachine()
 	}
-	panic(fmt.Errorf("unknown unit assignment policy: %q", policy))
+	return fmt.Errorf("unknown unit assignment policy: %q", policy)
 }
 
 // StartSync forces watchers to resynchronize their state with the
 // database immediately. This will happen periodically automatically.
 func (st *State) StartSync() {
 	st.watcher.StartSync()
-	st.pwatcher.StartSync()
-}
-
-// Sync forces watchers to resynchronize their state with the
-// database immediately, and waits until all events are known.
-func (st *State) Sync() {
-	st.watcher.Sync()
 	st.pwatcher.Sync()
 }
 
@@ -1011,6 +1085,30 @@ type cleanupDoc struct {
 	Prefix string
 }
 
+// newCleanupOp returns a txn.Op that creates a cleanup document with a unique
+// id and the supplied kind and prefix.
+func (st *State) newCleanupOp(kind, prefix string) txn.Op {
+	doc := &cleanupDoc{
+		Id:     bson.NewObjectId(),
+		Kind:   kind,
+		Prefix: prefix,
+	}
+	return txn.Op{
+		C:      st.cleanups.Name,
+		Id:     doc.Id,
+		Insert: doc,
+	}
+}
+
+// NeedsCleanup returns true if documents previously marked for removal exist.
+func (st *State) NeedsCleanup() (bool, error) {
+	count, err := st.cleanups.Count()
+	if err != nil {
+		return false, err
+	}
+	return count > 0, nil
+}
+
 // Cleanup removes all documents that were previously marked for removal, if
 // any such exist. It should be called periodically by at least one element
 // of the system.
@@ -1018,32 +1116,25 @@ func (st *State) Cleanup() error {
 	doc := cleanupDoc{}
 	iter := st.cleanups.Find(nil).Iter()
 	for iter.Next(&doc) {
-		var c *mgo.Collection
-		var sel interface{}
+		var err error
 		switch doc.Kind {
 		case "settings":
-			c = st.settings
-			sel = D{{"_id", D{{"$regex", "^" + doc.Prefix}}}}
+			err = st.cleanupSettings(doc.Prefix)
+		case "units":
+			err = st.cleanupUnits(doc.Prefix)
 		default:
-			log.Warningf("state: ignoring unknown cleanup kind %q", doc.Kind)
-			continue
+			err = fmt.Errorf("unknown cleanup kind %q", doc.Kind)
 		}
-		if count, err := c.Find(sel).Count(); err != nil {
-			return fmt.Errorf("cannot detect cleanup targets: %v", err)
-		} else if count != 0 {
-			// Documents marked for cleanup are not otherwise referenced in the
-			// system, and will not be under watch, and are therefore safe to
-			// delete directly.
-			if _, err := c.RemoveAll(sel); err != nil {
-				return fmt.Errorf("cannot remove documents marked for cleanup: %v", err)
-			}
+		if err != nil {
+			logger.Warningf("cleanup failed: %v", err)
+			continue
 		}
 		ops := []txn.Op{{
 			C:      st.cleanups.Name,
 			Id:     doc.Id,
 			Remove: true,
 		}}
-		if err := st.runner.Run(ops, "", nil); err != nil {
+		if err := st.runTransaction(ops); err != nil {
 			return fmt.Errorf("cannot remove empty cleanup document: %v", err)
 		}
 	}
@@ -1053,11 +1144,50 @@ func (st *State) Cleanup() error {
 	return nil
 }
 
+func (st *State) cleanupSettings(prefix string) error {
+	// Documents marked for cleanup are not otherwise referenced in the
+	// system, and will not be under watch, and are therefore safe to
+	// delete directly.
+	sel := D{{"_id", D{{"$regex", "^" + prefix}}}}
+	if count, err := st.settings.Find(sel).Count(); err != nil {
+		return fmt.Errorf("cannot detect cleanup targets: %v", err)
+	} else if count != 0 {
+		if _, err := st.settings.RemoveAll(sel); err != nil {
+			return fmt.Errorf("cannot remove documents marked for cleanup: %v", err)
+		}
+	}
+	return nil
+}
+
+func (st *State) cleanupUnits(prefix string) error {
+	// This won't miss units, because a Dying service cannot have units added
+	// to it. But we do have to remove the units themselves via individual
+	// transactions, because they could be in any state at all.
+	unit := &Unit{st: st}
+	sel := D{{"_id", D{{"$regex", "^" + prefix}}}, {"life", Alive}}
+	iter := st.units.Find(sel).Iter()
+	for iter.Next(&unit.doc) {
+		if err := unit.Destroy(); err != nil {
+			return err
+		}
+	}
+	if err := iter.Err(); err != nil {
+		return fmt.Errorf("cannot read unit document: %v", err)
+	}
+	return nil
+}
+
+// ResumeTransactions resumes all pending transactions.
+func (st *State) ResumeTransactions() error {
+	return st.runner.ResumeAll()
+}
+
 var tagPrefix = map[byte]string{
-	'm': "machine-",
-	's': "service-",
-	'u': "unit-",
-	'e': "environment-",
+	'm': names.MachineTagKind + "-",
+	's': names.ServiceTagKind + "-",
+	'u': names.UnitTagKind + "-",
+	'e': names.EnvironTagKind + "-",
+	'r': names.RelationTagKind + "-",
 }
 
 func tagForGlobalKey(key string) (string, bool) {
