@@ -1,7 +1,11 @@
+// Copyright 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package statecmd_test
 
 import (
-	. "launchpad.net/gocheck"
+	gc "launchpad.net/gocheck"
+
 	"launchpad.net/juju-core/juju/testing"
 	"launchpad.net/juju-core/state"
 	"launchpad.net/juju-core/state/api/params"
@@ -12,7 +16,7 @@ type DestroySuite struct {
 	testing.JujuConnSuite
 }
 
-var _ = Suite(&DestroySuite{})
+var _ = gc.Suite(&DestroySuite{})
 
 var serviceDestroyTests = []struct {
 	about   string
@@ -35,11 +39,11 @@ var serviceDestroyTests = []struct {
 	},
 }
 
-func (s *DestroySuite) TestServiceDestroy(c *C) {
+func (s *DestroySuite) TestServiceDestroy(c *gc.C) {
 	charm := s.AddTestingCharm(c, "dummy")
 	svc, err := s.State.AddService("dummy-service", charm)
-	c.Assert(err, IsNil)
-	c.Assert(svc.Life(), Equals, state.Alive)
+	c.Assert(err, gc.IsNil)
+	c.Assert(svc.Life(), gc.Equals, state.Alive)
 	c.Logf("Svc: %+v", svc)
 
 	for i, t := range serviceDestroyTests {
@@ -48,9 +52,9 @@ func (s *DestroySuite) TestServiceDestroy(c *C) {
 			ServiceName: t.service,
 		})
 		if t.err != "" {
-			c.Assert(err, ErrorMatches, t.err)
+			c.Assert(err, gc.ErrorMatches, t.err)
 		} else {
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 		}
 	}
 }
