@@ -1,10 +1,15 @@
+// Copyright 2012, 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package testing
 
 import (
 	"archive/tar"
 	"bytes"
 	"compress/gzip"
+	"crypto/sha256"
 	"fmt"
+	"io"
 	"os"
 	"time"
 )
@@ -44,10 +49,11 @@ func NewTarFile(name string, mode os.FileMode, contents string) *TarFile {
 	}
 }
 
-// TarGz returns the given files in gzipped tar-archive format.
-func TarGz(files ...*TarFile) []byte {
+// TarGz returns the given files in gzipped tar-archive format, along with the sha256 checksum.
+func TarGz(files ...*TarFile) ([]byte, string) {
 	var buf bytes.Buffer
-	gzw := gzip.NewWriter(&buf)
+	sha256hash := sha256.New()
+	gzw := gzip.NewWriter(io.MultiWriter(&buf, sha256hash))
 	tarw := tar.NewWriter(gzw)
 
 	for _, f := range files {
@@ -68,5 +74,6 @@ func TarGz(files ...*TarFile) []byte {
 	if err != nil {
 		panic(err)
 	}
-	return buf.Bytes()
+	checksum := fmt.Sprintf("%x", sha256hash.Sum(nil))
+	return buf.Bytes(), checksum
 }
