@@ -1,15 +1,21 @@
+// Copyright 2012, 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package firewaller
 
 import (
 	"fmt"
+
+	"launchpad.net/tomb"
+
 	"launchpad.net/juju-core/environs"
 	"launchpad.net/juju-core/environs/config"
+	"launchpad.net/juju-core/errors"
+	"launchpad.net/juju-core/instance"
 	"launchpad.net/juju-core/log"
 	"launchpad.net/juju-core/state"
-	"launchpad.net/juju-core/state/api/params"
 	"launchpad.net/juju-core/state/watcher"
 	"launchpad.net/juju-core/worker"
-	"launchpad.net/tomb"
 )
 
 // Firewaller watches the state for ports opened or closed
@@ -18,8 +24,8 @@ type Firewaller struct {
 	tomb            tomb.Tomb
 	st              *state.State
 	environ         environs.Environ
-	environWatcher  *state.EnvironConfigWatcher
-	machinesWatcher *state.LifecycleWatcher
+	environWatcher  state.NotifyWatcher
+	machinesWatcher state.StringsWatcher
 	machineds       map[string]*machineData
 	unitsChange     chan *unitsChange
 	unitds          map[string]*unitData
@@ -27,15 +33,15 @@ type Firewaller struct {
 	serviceds       map[string]*serviceData
 	exposedChange   chan *exposedChange
 	globalMode      bool
-	globalPortRef   map[params.Port]int
+	globalPortRef   map[instance.Port]int
 }
 
 // NewFirewaller returns a new Firewaller.
 func NewFirewaller(st *state.State) *Firewaller {
 	fw := &Firewaller{
 		st:              st,
-		environWatcher:  st.WatchEnvironConfig(),
-		machinesWatcher: st.WatchMachines(),
+		environWatcher:  st.WatchForEnvironConfigChanges(),
+		machinesWatcher: st.WatchEnvironMachines(),
 		machineds:       make(map[string]*machineData),
 		unitsChange:     make(chan *unitsChange),
 		unitds:          make(map[string]*unitData),
@@ -56,23 +62,27 @@ func (fw *Firewaller) loop() error {
 	var err error
 	var reconciled bool
 
-	fw.environ, err = worker.WaitForEnviron(fw.environWatcher, fw.tomb.Dying())
+	fw.environ, err = worker.WaitForEnviron(fw.environWatcher, fw.st, fw.tomb.Dying())
 	if err != nil {
 		return err
 	}
 	if fw.environ.Config().FirewallMode() == config.FwGlobal {
 		fw.globalMode = true
-		fw.globalPortRef = make(map[params.Port]int)
+		fw.globalPortRef = make(map[instance.Port]int)
 	}
 	for {
 		select {
 		case <-fw.tomb.Dying():
 			return tomb.ErrDying
-		case change, ok := <-fw.environWatcher.Changes():
+		case _, ok := <-fw.environWatcher.Changes():
 			if !ok {
 				return watcher.MustErr(fw.environWatcher)
 			}
-			if err := fw.environ.SetConfig(change); err != nil {
+			config, err := fw.st.EnvironConfig()
+			if err != nil {
+				return err
+			}
+			if err := fw.environ.SetConfig(config); err != nil {
 				log.Errorf("worker/firewaller: loaded invalid environment configuration: %v", err)
 			}
 		case change, ok := <-fw.machinesWatcher.Changes():
@@ -114,7 +124,6 @@ func (fw *Firewaller) loop() error {
 			}
 		}
 	}
-	panic("not reached")
 }
 
 // stop a watcher with logging of a possible error.
@@ -131,10 +140,10 @@ func (fw *Firewaller) startMachine(id string) error {
 		fw:     fw,
 		id:     id,
 		unitds: make(map[string]*unitData),
-		ports:  make([]params.Port, 0),
+		ports:  make([]instance.Port, 0),
 	}
 	m, err := machined.machine()
-	if state.IsNotFound(err) {
+	if errors.IsNotFoundError(err) {
 		return nil
 	} else if err != nil {
 		return fmt.Errorf("worker/firewaller: cannot watch machine units: %v", err)
@@ -153,7 +162,7 @@ func (fw *Firewaller) startMachine(id string) error {
 		err = fw.unitsChanged(&unitsChange{machined, change})
 		if err != nil {
 			stop("units watcher", unitw)
-			return fmt.Errorf("worker/firewaller: start watching machine %d faild: %v", id, err)
+			return fmt.Errorf("worker/firewaller: cannot respond to units changes for machine %q: %v", id, err)
 		}
 	}
 	go machined.watchLoop(unitw)
@@ -190,7 +199,7 @@ func (fw *Firewaller) startUnit(unit *state.Unit, machineId string) error {
 	unitd.serviced = fw.serviceds[serviceName]
 	unitd.serviced.unitds[unitName] = unitd
 
-	ports := make([]params.Port, len(unitd.ports))
+	ports := make([]instance.Port, len(unitd.ports))
 	copy(ports, unitd.ports)
 
 	go unitd.watchLoop(ports)
@@ -219,7 +228,7 @@ func (fw *Firewaller) reconcileGlobal() error {
 	if err != nil {
 		return err
 	}
-	collector := make(map[params.Port]bool)
+	collector := make(map[instance.Port]bool)
 	for _, unitd := range fw.unitds {
 		if unitd.serviced.exposed {
 			for _, port := range unitd.ports {
@@ -227,13 +236,13 @@ func (fw *Firewaller) reconcileGlobal() error {
 			}
 		}
 	}
-	wantedPorts := []params.Port{}
+	wantedPorts := []instance.Port{}
 	for port := range collector {
 		wantedPorts = append(wantedPorts, port)
 	}
 	// Check which ports to open or to close.
-	toOpen := diff(wantedPorts, initialPorts)
-	toClose := diff(initialPorts, wantedPorts)
+	toOpen := Diff(wantedPorts, initialPorts)
+	toClose := Diff(initialPorts, wantedPorts)
 	if len(toOpen) > 0 {
 		log.Infof("worker/firewaller: opening global ports %v", toOpen)
 		if err := fw.environ.OpenPorts(toOpen); err != nil {
@@ -257,7 +266,7 @@ func (fw *Firewaller) reconcileGlobal() error {
 func (fw *Firewaller) reconcileInstances() error {
 	for _, machined := range fw.machineds {
 		m, err := machined.machine()
-		if state.IsNotFound(err) {
+		if errors.IsNotFoundError(err) {
 			if err := fw.forgetMachine(machined); err != nil {
 				return err
 			}
@@ -265,11 +274,11 @@ func (fw *Firewaller) reconcileInstances() error {
 		} else if err != nil {
 			return err
 		}
-		instanceId, ok := m.InstanceId()
-		if !ok {
-			return state.NotFoundf("instance id for %v", m)
+		instanceId, err := m.InstanceId()
+		if err != nil {
+			return err
 		}
-		instances, err := fw.environ.Instances([]state.InstanceId{instanceId})
+		instances, err := fw.environ.Instances([]instance.Id{instanceId})
 		if err == environs.ErrNoInstances {
 			return nil
 		} else if err != nil {
@@ -280,8 +289,8 @@ func (fw *Firewaller) reconcileInstances() error {
 			return err
 		}
 		// Check which ports to open or to close.
-		toOpen := diff(machined.ports, initialPorts)
-		toClose := diff(initialPorts, machined.ports)
+		toOpen := Diff(machined.ports, initialPorts)
+		toClose := Diff(initialPorts, machined.ports)
 		if len(toOpen) > 0 {
 			log.Infof("worker/firewaller: opening instance ports %v for machine %s",
 				toOpen, machined.id)
@@ -309,13 +318,13 @@ func (fw *Firewaller) unitsChanged(change *unitsChange) error {
 	changed := []*unitData{}
 	for _, name := range change.units {
 		unit, err := fw.st.Unit(name)
-		if err != nil && !state.IsNotFound(err) {
+		if err != nil && !errors.IsNotFoundError(err) {
 			return err
 		}
 		var machineId string
 		if unit != nil {
 			machineId, err = unit.AssignedMachineId()
-			if state.IsNotFound(err) {
+			if errors.IsNotFoundError(err) {
 				continue
 			} else if err != nil && !state.IsNotAssigned(err) {
 				return err
@@ -360,7 +369,7 @@ func (fw *Firewaller) flushUnits(unitds []*unitData) error {
 // flushMachine opens and closes ports for the passed machine.
 func (fw *Firewaller) flushMachine(machined *machineData) error {
 	// Gather ports to open and close.
-	ports := map[params.Port]bool{}
+	ports := map[instance.Port]bool{}
 	for _, unitd := range machined.unitds {
 		if unitd.serviced.exposed {
 			for _, port := range unitd.ports {
@@ -368,12 +377,12 @@ func (fw *Firewaller) flushMachine(machined *machineData) error {
 			}
 		}
 	}
-	want := []params.Port{}
+	want := []instance.Port{}
 	for port := range ports {
 		want = append(want, port)
 	}
-	toOpen := diff(want, machined.ports)
-	toClose := diff(machined.ports, want)
+	toOpen := Diff(want, machined.ports)
+	toClose := Diff(machined.ports, want)
 	machined.ports = want
 	if fw.globalMode {
 		return fw.flushGlobalPorts(toOpen, toClose)
@@ -384,9 +393,9 @@ func (fw *Firewaller) flushMachine(machined *machineData) error {
 // flushGlobalPorts opens and closes global ports in the environment.
 // It keeps a reference count for ports so that only 0-to-1 and 1-to-0 events
 // modify the environment.
-func (fw *Firewaller) flushGlobalPorts(rawOpen, rawClose []params.Port) error {
+func (fw *Firewaller) flushGlobalPorts(rawOpen, rawClose []instance.Port) error {
 	// Filter which ports are really to open or close.
-	var toOpen, toClose []params.Port
+	var toOpen, toClose []instance.Port
 	for _, port := range rawOpen {
 		if fw.globalPortRef[port] == 0 {
 			toOpen = append(toOpen, port)
@@ -421,7 +430,7 @@ func (fw *Firewaller) flushGlobalPorts(rawOpen, rawClose []params.Port) error {
 }
 
 // flushGlobalPorts opens and closes ports global on the machine.
-func (fw *Firewaller) flushInstancePorts(machined *machineData, toOpen, toClose []params.Port) error {
+func (fw *Firewaller) flushInstancePorts(machined *machineData, toOpen, toClose []instance.Port) error {
 	// If there's nothing to do, do nothing.
 	// This is important because when a machine is first created,
 	// it will have no instance id but also no open ports -
@@ -430,17 +439,17 @@ func (fw *Firewaller) flushInstancePorts(machined *machineData, toOpen, toClose
 		return nil
 	}
 	m, err := machined.machine()
-	if state.IsNotFound(err) {
+	if errors.IsNotFoundError(err) {
 		return nil
 	}
 	if err != nil {
 		return err
 	}
-	instanceId, ok := m.InstanceId()
-	if !ok {
-		return state.NotFoundf("instance id for %v", m)
+	instanceId, err := m.InstanceId()
+	if err != nil {
+		return err
 	}
-	instances, err := fw.environ.Instances([]state.InstanceId{instanceId})
+	instances, err := fw.environ.Instances([]instance.Id{instanceId})
 	if err != nil {
 		return err
 	}
@@ -469,7 +478,7 @@ func (fw *Firewaller) flushInstancePorts(machined *machineData, toOpen, toClose
 // machines that are dying.
 func (fw *Firewaller) machineLifeChanged(id string) error {
 	m, err := fw.st.Machine(id)
-	found := !state.IsNotFound(err)
+	found := !errors.IsNotFoundError(err)
 	if found && err != nil {
 		return err
 	}
@@ -550,7 +559,12 @@ func (fw *Firewaller) Err() (reason error) {
 	return fw.tomb.Err()
 }
 
-// Wait waits for the Firewaller to exit.
+// Kill implements worker.Worker.Kill.
+func (fw *Firewaller) Kill() {
+	fw.tomb.Kill(nil)
+}
+
+// Wait implements worker.Worker.Wait.
 func (fw *Firewaller) Wait() error {
 	return fw.tomb.Wait()
 }
@@ -573,7 +587,7 @@ type machineData struct {
 	fw     *Firewaller
 	id     string
 	unitds map[string]*unitData
-	ports  []params.Port
+	ports  []instance.Port
 }
 
 func (md *machineData) machine() (*state.Machine, error) {
@@ -581,7 +595,7 @@ func (md *machineData) machine() (*state.Machine, error) {
 }
 
 // watchLoop watches the machine for units added or removed.
-func (md *machineData) watchLoop(unitw *state.MachineUnitsWatcher) {
+func (md *machineData) watchLoop(unitw state.StringsWatcher) {
 	defer md.tomb.Done()
 	defer watcher.Stop(unitw, &md.tomb)
 	for {
@@ -591,7 +605,7 @@ func (md *machineData) watchLoop(unitw *state.MachineUnitsWatcher) {
 		case change, ok := <-unitw.Changes():
 			if !ok {
 				_, err := md.machine()
-				if !state.IsNotFound(err) {
+				if !errors.IsNotFoundError(err) {
 					md.fw.tomb.Kill(watcher.MustErr(unitw))
 				}
 				return
@@ -614,7 +628,7 @@ func (md *machineData) Stop() error {
 // portsChange contains the changed ports for one specific unit.
 type portsChange struct {
 	unitd *unitData
-	ports []params.Port
+	ports []instance.Port
 }
 
 // unitData holds unit details and watches port changes.
@@ -624,11 +638,11 @@ type unitData struct {
 	unit     *state.Unit
 	serviced *serviceData
 	machined *machineData
-	ports    []params.Port
+	ports    []instance.Port
 }
 
 // watchLoop watches the unit for port changes.
-func (ud *unitData) watchLoop(latestPorts []params.Port) {
+func (ud *unitData) watchLoop(latestPorts []instance.Port) {
 	defer ud.tomb.Done()
 	w := ud.unit.Watch()
 	defer watcher.Stop(w, &ud.tomb)
@@ -642,7 +656,7 @@ func (ud *unitData) watchLoop(latestPorts []params.Port) {
 				return
 			}
 			if err := ud.unit.Refresh(); err != nil {
-				if !state.IsNotFound(err) {
+				if !errors.IsNotFoundError(err) {
 					ud.fw.tomb.Kill(err)
 				}
 				return
@@ -663,7 +677,7 @@ func (ud *unitData) watchLoop(latestPorts []params.Port) {
 
 // samePorts returns whether old and new contain the same set of ports.
 // Both old and new must be sorted.
-func samePorts(old, new []params.Port) bool {
+func samePorts(old, new []instance.Port) bool {
 	if len(old) != len(new) {
 		return false
 	}
@@ -711,7 +725,7 @@ func (sd *serviceData) watchLoop(exposed bool) {
 				return
 			}
 			if err := sd.service.Refresh(); err != nil {
-				if !state.IsNotFound(err) {
+				if !errors.IsNotFoundError(err) {
 					sd.fw.tomb.Kill(err)
 				}
 				return
@@ -736,8 +750,8 @@ func (sd *serviceData) Stop() error {
 	return sd.tomb.Wait()
 }
 
-// diff returns all the ports that exist in A but not B.
-func diff(A, B []params.Port) (missing []params.Port) {
+// Diff returns all the ports that exist in A but not B.
+func Diff(A, B []instance.Port) (missing []instance.Port) {
 next:
 	for _, a := range A {
 		for _, b := range B {
