@@ -1,26 +1,34 @@
+// Copyright 2012, 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package uniter_test
 
 import (
 	"fmt"
 	"io/ioutil"
-	. "launchpad.net/gocheck"
-	"launchpad.net/juju-core/juju/testing"
-	"launchpad.net/juju-core/log"
-	"launchpad.net/juju-core/state"
-	"launchpad.net/juju-core/utils"
-	"launchpad.net/juju-core/worker/uniter"
-	"launchpad.net/juju-core/worker/uniter/jujuc"
 	"os"
 	"path/filepath"
 	"strings"
 	"time"
+
+	gc "launchpad.net/gocheck"
+
+	"launchpad.net/juju-core/charm"
+	"launchpad.net/juju-core/juju/testing"
+	"launchpad.net/juju-core/state"
+	"launchpad.net/juju-core/state/api"
+	"launchpad.net/juju-core/state/api/params"
+	apiuniter "launchpad.net/juju-core/state/api/uniter"
+	"launchpad.net/juju-core/utils"
+	"launchpad.net/juju-core/worker/uniter"
+	"launchpad.net/juju-core/worker/uniter/jujuc"
 )
 
 type RunHookSuite struct {
 	HookContextSuite
 }
 
-var _ = Suite(&RunHookSuite{})
+var _ = gc.Suite(&RunHookSuite{})
 
 type hookSpec struct {
 	// name is the name of the hook.
@@ -42,14 +50,14 @@ type hookSpec struct {
 // the charm will write it to stdout and stderr, with each one prefixed
 // by name of the stream.  It returns the charm directory and the path
 // to which the hook script will write environment variables.
-func makeCharm(c *C, spec hookSpec) (charmDir, outPath string) {
+func makeCharm(c *gc.C, spec hookSpec) (charmDir, outPath string) {
 	charmDir = c.MkDir()
 	hooksDir := filepath.Join(charmDir, "hooks")
 	err := os.Mkdir(hooksDir, 0755)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	c.Logf("openfile perm %v", spec.perm)
 	hook, err := os.OpenFile(filepath.Join(hooksDir, spec.name), os.O_CREATE|os.O_WRONLY, spec.perm)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	defer hook.Close()
 	printf := func(f string, a ...interface{}) {
 		if _, err := fmt.Fprintf(hook, f+"\n", a...); err != nil {
@@ -77,7 +85,7 @@ func makeCharm(c *C, spec hookSpec) (charmDir, outPath string) {
 	return charmDir, outPath
 }
 
-func AssertEnvContains(c *C, lines []string, env map[string]string) {
+func AssertEnvContains(c *gc.C, lines []string, env map[string]string) {
 	for k, v := range env {
 		sought := k + "=" + v
 		found := false
@@ -87,14 +95,14 @@ func AssertEnvContains(c *C, lines []string, env map[string]string) {
 				continue
 			}
 		}
-		comment := Commentf("expected to find %v among %v", sought, lines)
-		c.Assert(found, Equals, true, comment)
+		comment := gc.Commentf("expected to find %v among %v", sought, lines)
+		c.Assert(found, gc.Equals, true, comment)
 	}
 }
 
-func AssertEnv(c *C, outPath string, charmDir string, env map[string]string, uuid string) {
+func AssertEnv(c *gc.C, outPath string, charmDir string, env map[string]string, uuid string) {
 	out, err := ioutil.ReadFile(outPath)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	lines := strings.Split(string(out), "\n")
 	AssertEnvContains(c, lines, env)
 	AssertEnvContains(c, lines, map[string]string{
@@ -110,6 +118,9 @@ func AssertEnv(c *C, outPath string, charmDir string, env map[string]string, uui
 // the bufio line reader.
 const lineBufferSize = 4096
 
+var apiAddrs = []string{"a1:123", "a2:123"}
+var expectedApiAddrs = strings.Join(apiAddrs, " ")
+
 var runHookTests = []struct {
 	summary string
 	relid   int
@@ -162,17 +173,19 @@ var runHookTests = []struct {
 		relid:   -1,
 		spec:    hookSpec{perm: 0700},
 		env: map[string]string{
-			"JUJU_UNIT_NAME": "u/0",
+			"JUJU_UNIT_NAME":     "u/0",
+			"JUJU_API_ADDRESSES": expectedApiAddrs,
 		},
 	}, {
 		summary: "check shell environment for relation-broken hook context",
 		relid:   1,
 		spec:    hookSpec{perm: 0700},
 		env: map[string]string{
-			"JUJU_UNIT_NAME":   "u/0",
-			"JUJU_RELATION":    "db",
-			"JUJU_RELATION_ID": "db:1",
-			"JUJU_REMOTE_UNIT": "",
+			"JUJU_UNIT_NAME":     "u/0",
+			"JUJU_API_ADDRESSES": expectedApiAddrs,
+			"JUJU_RELATION":      "db",
+			"JUJU_RELATION_ID":   "db:1",
+			"JUJU_REMOTE_UNIT":   "",
 		},
 	}, {
 		summary: "check shell environment for relation hook context",
@@ -180,33 +193,18 @@ var runHookTests = []struct {
 		remote:  "r/1",
 		spec:    hookSpec{perm: 0700},
 		env: map[string]string{
-			"JUJU_UNIT_NAME":   "u/0",
-			"JUJU_RELATION":    "db",
-			"JUJU_RELATION_ID": "db:1",
-			"JUJU_REMOTE_UNIT": "r/1",
+			"JUJU_UNIT_NAME":     "u/0",
+			"JUJU_API_ADDRESSES": expectedApiAddrs,
+			"JUJU_RELATION":      "db",
+			"JUJU_RELATION_ID":   "db:1",
+			"JUJU_REMOTE_UNIT":   "r/1",
 		},
 	},
 }
 
-type logRecorder struct {
-	c      *C
-	prefix string
-	lines  []string
-}
-
-func (l *logRecorder) Output(calldepth int, s string) error {
-	if strings.HasPrefix(s, l.prefix) {
-		l.lines = append(l.lines, s[len(l.prefix):])
-	}
-	l.c.Logf("%s", s)
-	return nil
-}
-
-func (s *RunHookSuite) TestRunHook(c *C) {
-	logger := &logRecorder{c: c, prefix: "INFO worker/uniter: HOOK "}
-	defer log.SetTarget(log.SetTarget(logger))
+func (s *RunHookSuite) TestRunHook(c *gc.C) {
 	uuid, err := utils.NewUUID()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	for i, t := range runHookTests {
 		c.Logf("test %d: %s; perm %v", i, t.summary, t.spec.perm)
 		ctx := s.GetHookContext(c, uuid.String(), t.relid, t.remote)
@@ -220,13 +218,12 @@ func (s *RunHookSuite) TestRunHook(c *C) {
 			charmDir, outPath = makeCharm(c, spec)
 		}
 		toolsDir := c.MkDir()
-		logger.lines = nil
 		t0 := time.Now()
 		err := ctx.RunHook("something-happened", charmDir, toolsDir, "/path/to/socket")
 		if t.err == "" {
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 		} else {
-			c.Assert(err, ErrorMatches, t.err)
+			c.Assert(err, gc.ErrorMatches, t.err)
 		}
 		if t.env != nil {
 			env := map[string]string{"PATH": toolsDir + ":" + os.Getenv("PATH")}
@@ -235,17 +232,9 @@ func (s *RunHookSuite) TestRunHook(c *C) {
 			}
 			AssertEnv(c, outPath, charmDir, env, uuid.String())
 		}
-		var expectLog []string
-		if t.spec.stdout != "" {
-			expectLog = append(expectLog, splitLine(t.spec.stdout)...)
-		}
-		if t.spec.stderr != "" {
-			expectLog = append(expectLog, splitLine(t.spec.stderr)...)
-		}
 		if t.spec.background != "" && time.Now().Sub(t0) > 5*time.Second {
 			c.Errorf("background process holding up hook execution")
 		}
-		c.Assert(logger.lines, DeepEquals, expectLog)
 	}
 }
 
@@ -262,10 +251,10 @@ func splitLine(s string) []string {
 	return ss
 }
 
-func (s *RunHookSuite) TestRunHookRelationFlushing(c *C) {
+func (s *RunHookSuite) TestRunHookRelationFlushing(c *gc.C) {
 	// Create a charm with a breaking hook.
 	uuid, err := utils.NewUUID()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	ctx := s.GetHookContext(c, uuid.String(), -1, "")
 	charmDir, _ := makeCharm(c, hookSpec{
 		name: "something-happened",
@@ -275,63 +264,69 @@ func (s *RunHookSuite) TestRunHookRelationFlushing(c *C) {
 
 	// Mess with multiple relation settings.
 	node0, err := s.relctxs[0].Settings()
-	node0.Set("foo", 1)
+	node0.Set("foo", "1")
 	node1, err := s.relctxs[1].Settings()
-	node1.Set("bar", 2)
+	node1.Set("bar", "2")
 
 	// Run the failing hook.
 	err = ctx.RunHook("something-happened", charmDir, c.MkDir(), "/path/to/socket")
-	c.Assert(err, ErrorMatches, "exit status 123")
+	c.Assert(err, gc.ErrorMatches, "exit status 123")
 
 	// Check that the changes to the local settings nodes have been discarded.
 	node0, err = s.relctxs[0].Settings()
-	c.Assert(err, IsNil)
-	c.Assert(node0.Map(), DeepEquals, map[string]interface{}{"relation-name": "db0"})
+	c.Assert(err, gc.IsNil)
+	c.Assert(node0.Map(), gc.DeepEquals, params.RelationSettings{"relation-name": "db0"})
 	node1, err = s.relctxs[1].Settings()
-	c.Assert(err, IsNil)
-	c.Assert(node1.Map(), DeepEquals, map[string]interface{}{"relation-name": "db1"})
+	c.Assert(err, gc.IsNil)
+	c.Assert(node1.Map(), gc.DeepEquals, params.RelationSettings{"relation-name": "db1"})
 
 	// Check that the changes have been written to state.
 	settings0, err := s.relunits[0].ReadSettings("u/0")
-	c.Assert(err, IsNil)
-	c.Assert(settings0, DeepEquals, node0.Map())
+	c.Assert(err, gc.IsNil)
+	c.Assert(settings0, gc.DeepEquals, map[string]interface{}{"relation-name": "db0"})
 	settings1, err := s.relunits[1].ReadSettings("u/0")
-	c.Assert(err, IsNil)
-	c.Assert(settings1, DeepEquals, node1.Map())
+	c.Assert(err, gc.IsNil)
+	c.Assert(settings1, gc.DeepEquals, map[string]interface{}{"relation-name": "db1"})
 
 	// Create a charm with a working hook, and mess with settings again.
 	charmDir, _ = makeCharm(c, hookSpec{
 		name: "something-happened",
 		perm: 0700,
 	})
-	node0.Set("baz", 3)
-	node1.Set("qux", 4)
+	node0.Set("baz", "3")
+	node1.Set("qux", "4")
 
 	// Run the hook.
 	err = ctx.RunHook("something-happened", charmDir, c.MkDir(), "/path/to/socket")
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 
 	// Check that the changes to the local settings nodes are still there.
 	node0, err = s.relctxs[0].Settings()
-	c.Assert(err, IsNil)
-	c.Assert(node0.Map(), DeepEquals, map[string]interface{}{
+	c.Assert(err, gc.IsNil)
+	c.Assert(node0.Map(), gc.DeepEquals, params.RelationSettings{
 		"relation-name": "db0",
-		"baz":           3,
+		"baz":           "3",
 	})
 	node1, err = s.relctxs[1].Settings()
-	c.Assert(err, IsNil)
-	c.Assert(node1.Map(), DeepEquals, map[string]interface{}{
+	c.Assert(err, gc.IsNil)
+	c.Assert(node1.Map(), gc.DeepEquals, params.RelationSettings{
 		"relation-name": "db1",
-		"qux":           4,
+		"qux":           "4",
 	})
 
 	// Check that the changes have been written to state.
 	settings0, err = s.relunits[0].ReadSettings("u/0")
-	c.Assert(err, IsNil)
-	c.Assert(settings0, DeepEquals, node0.Map())
+	c.Assert(err, gc.IsNil)
+	c.Assert(settings0, gc.DeepEquals, map[string]interface{}{
+		"relation-name": "db0",
+		"baz":           "3",
+	})
 	settings1, err = s.relunits[1].ReadSettings("u/0")
-	c.Assert(err, IsNil)
-	c.Assert(settings1, DeepEquals, node1.Map())
+	c.Assert(err, gc.IsNil)
+	c.Assert(settings1, gc.DeepEquals, map[string]interface{}{
+		"relation-name": "db1",
+		"qux":           "4",
+	})
 }
 
 type ContextRelationSuite struct {
@@ -339,269 +334,289 @@ type ContextRelationSuite struct {
 	svc *state.Service
 	rel *state.Relation
 	ru  *state.RelationUnit
+
+	st         *api.State
+	uniter     *apiuniter.State
+	apiRelUnit *apiuniter.RelationUnit
 }
 
-var _ = Suite(&ContextRelationSuite{})
+var _ = gc.Suite(&ContextRelationSuite{})
 
-func (s *ContextRelationSuite) SetUpTest(c *C) {
+func (s *ContextRelationSuite) SetUpTest(c *gc.C) {
 	s.JujuConnSuite.SetUpTest(c)
 	ch := s.AddTestingCharm(c, "riak")
 	var err error
 	s.svc, err = s.State.AddService("u", ch)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	rels, err := s.svc.Relations()
-	c.Assert(err, IsNil)
-	c.Assert(rels, HasLen, 1)
+	c.Assert(err, gc.IsNil)
+	c.Assert(rels, gc.HasLen, 1)
 	s.rel = rels[0]
 	unit, err := s.svc.AddUnit()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	s.ru, err = s.rel.Unit(unit)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	err = s.ru.EnterScope(nil)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
+
+	err = unit.SetPassword("password")
+	c.Assert(err, gc.IsNil)
+	s.st = s.OpenAPIAs(c, unit.Tag(), "password")
+	s.uniter = s.st.Uniter()
+	c.Assert(s.uniter, gc.NotNil)
+
+	apiRel, err := s.uniter.Relation(s.rel.Tag())
+	c.Assert(err, gc.IsNil)
+	apiUnit, err := s.uniter.Unit(unit.Tag())
+	c.Assert(err, gc.IsNil)
+	s.apiRelUnit, err = apiRel.Unit(apiUnit)
+	c.Assert(err, gc.IsNil)
 }
 
-func (s *ContextRelationSuite) TestChangeMembers(c *C) {
-	ctx := uniter.NewContextRelation(s.ru, nil)
-	c.Assert(ctx.UnitNames(), HasLen, 0)
+func (s *ContextRelationSuite) TestChangeMembers(c *gc.C) {
+	ctx := uniter.NewContextRelation(s.apiRelUnit, nil)
+	c.Assert(ctx.UnitNames(), gc.HasLen, 0)
 
 	// Check the units and settings after a simple update.
 	ctx.UpdateMembers(uniter.SettingsMap{
-		"u/2": {"baz": 2},
-		"u/4": {"qux": 4},
+		"u/2": {"baz": "2"},
+		"u/4": {"qux": "4"},
 	})
-	c.Assert(ctx.UnitNames(), DeepEquals, []string{"u/2", "u/4"})
-	assertSettings := func(unit string, expect map[string]interface{}) {
+	c.Assert(ctx.UnitNames(), gc.DeepEquals, []string{"u/2", "u/4"})
+	assertSettings := func(unit string, expect params.RelationSettings) {
 		actual, err := ctx.ReadSettings(unit)
-		c.Assert(err, IsNil)
-		c.Assert(actual, DeepEquals, expect)
+		c.Assert(err, gc.IsNil)
+		c.Assert(actual, gc.DeepEquals, expect)
 	}
-	assertSettings("u/2", map[string]interface{}{"baz": 2})
-	assertSettings("u/4", map[string]interface{}{"qux": 4})
+	assertSettings("u/2", params.RelationSettings{"baz": "2"})
+	assertSettings("u/4", params.RelationSettings{"qux": "4"})
 
 	// Send a second update; check that members are only added, not removed.
 	ctx.UpdateMembers(uniter.SettingsMap{
-		"u/1": {"foo": 1},
-		"u/2": nil,
-		"u/3": {"bar": 3},
+		"u/1": {"foo": "1"},
+		"u/2": {"abc": "2"},
+		"u/3": {"bar": "3"},
 	})
-	c.Assert(ctx.UnitNames(), DeepEquals, []string{"u/1", "u/2", "u/3", "u/4"})
+	c.Assert(ctx.UnitNames(), gc.DeepEquals, []string{"u/1", "u/2", "u/3", "u/4"})
 
-	// Check that all settings remain cached, including u/2's (which lacked
-	// new settings data in the second update).
-	assertSettings("u/1", map[string]interface{}{"foo": 1})
-	assertSettings("u/2", map[string]interface{}{"baz": 2})
-	assertSettings("u/3", map[string]interface{}{"bar": 3})
-	assertSettings("u/4", map[string]interface{}{"qux": 4})
+	// Check that all settings remain cached.
+	assertSettings("u/1", params.RelationSettings{"foo": "1"})
+	assertSettings("u/2", params.RelationSettings{"abc": "2"})
+	assertSettings("u/3", params.RelationSettings{"bar": "3"})
+	assertSettings("u/4", params.RelationSettings{"qux": "4"})
 
 	// Delete a member, and check that it is no longer a member...
 	ctx.DeleteMember("u/2")
-	c.Assert(ctx.UnitNames(), DeepEquals, []string{"u/1", "u/3", "u/4"})
+	c.Assert(ctx.UnitNames(), gc.DeepEquals, []string{"u/1", "u/3", "u/4"})
 
 	// ...and that its settings are no longer cached.
 	_, err := ctx.ReadSettings("u/2")
-	c.Assert(err, ErrorMatches, `cannot read settings for unit "u/2" in relation "u:ring": settings not found`)
+	c.Assert(err, gc.ErrorMatches, "permission denied")
 }
 
-func (s *ContextRelationSuite) TestMemberCaching(c *C) {
+func (s *ContextRelationSuite) TestMemberCaching(c *gc.C) {
 	unit, err := s.svc.AddUnit()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	ru, err := s.rel.Unit(unit)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	err = ru.EnterScope(map[string]interface{}{"blib": "blob"})
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	settings, err := ru.Settings()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	settings.Set("ping", "pong")
 	_, err = settings.Write()
-	c.Assert(err, IsNil)
-	ctx := uniter.NewContextRelation(s.ru, map[string]int64{"u/1": 0})
+	c.Assert(err, gc.IsNil)
+	ctx := uniter.NewContextRelation(s.apiRelUnit, map[string]int64{"u/1": 0})
 
 	// Check that uncached settings are read from state.
 	m, err := ctx.ReadSettings("u/1")
-	c.Assert(err, IsNil)
-	expect := settings.Map()
-	c.Assert(m, DeepEquals, expect)
+	c.Assert(err, gc.IsNil)
+	expectMap := settings.Map()
+	expectSettings := convertMap(expectMap)
+	c.Assert(m, gc.DeepEquals, expectSettings)
 
 	// Check that changes to state do not affect the cached settings.
 	settings.Set("ping", "pow")
 	_, err = settings.Write()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	m, err = ctx.ReadSettings("u/1")
-	c.Assert(err, IsNil)
-	c.Assert(m, DeepEquals, expect)
+	c.Assert(err, gc.IsNil)
+	c.Assert(m, gc.DeepEquals, expectSettings)
 
 	// Check that ClearCache spares the members cache.
 	ctx.ClearCache()
 	m, err = ctx.ReadSettings("u/1")
-	c.Assert(err, IsNil)
-	c.Assert(m, DeepEquals, expect)
+	c.Assert(err, gc.IsNil)
+	c.Assert(m, gc.DeepEquals, expectSettings)
 
 	// Check that updating the context overwrites the cached settings, and
 	// that the contents of state are ignored.
 	ctx.UpdateMembers(uniter.SettingsMap{"u/1": {"entirely": "different"}})
 	m, err = ctx.ReadSettings("u/1")
-	c.Assert(err, IsNil)
-	c.Assert(m, DeepEquals, map[string]interface{}{"entirely": "different"})
+	c.Assert(err, gc.IsNil)
+	c.Assert(m, gc.DeepEquals, params.RelationSettings{"entirely": "different"})
 }
 
-func (s *ContextRelationSuite) TestNonMemberCaching(c *C) {
+func (s *ContextRelationSuite) TestNonMemberCaching(c *gc.C) {
 	unit, err := s.svc.AddUnit()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	ru, err := s.rel.Unit(unit)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	err = ru.EnterScope(map[string]interface{}{"blib": "blob"})
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	settings, err := ru.Settings()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	settings.Set("ping", "pong")
 	_, err = settings.Write()
-	c.Assert(err, IsNil)
-	ctx := uniter.NewContextRelation(s.ru, nil)
+	c.Assert(err, gc.IsNil)
+	ctx := uniter.NewContextRelation(s.apiRelUnit, nil)
 
 	// Check that settings are read from state.
 	m, err := ctx.ReadSettings("u/1")
-	c.Assert(err, IsNil)
-	expect := settings.Map()
-	c.Assert(m, DeepEquals, expect)
+	c.Assert(err, gc.IsNil)
+	expectMap := settings.Map()
+	expectSettings := convertMap(expectMap)
+	c.Assert(m, gc.DeepEquals, expectSettings)
 
 	// Check that changes to state do not affect the obtained settings...
 	settings.Set("ping", "pow")
 	_, err = settings.Write()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	m, err = ctx.ReadSettings("u/1")
-	c.Assert(err, IsNil)
-	c.Assert(m, DeepEquals, expect)
+	c.Assert(err, gc.IsNil)
+	c.Assert(m, gc.DeepEquals, expectSettings)
 
 	// ...until the caches are cleared.
 	ctx.ClearCache()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	m, err = ctx.ReadSettings("u/1")
-	c.Assert(err, IsNil)
-	c.Assert(m["ping"], Equals, "pow")
+	c.Assert(err, gc.IsNil)
+	c.Assert(m["ping"], gc.Equals, "pow")
 }
 
-func (s *ContextRelationSuite) TestSettings(c *C) {
-	ctx := uniter.NewContextRelation(s.ru, nil)
+func (s *ContextRelationSuite) TestSettings(c *gc.C) {
+	ctx := uniter.NewContextRelation(s.apiRelUnit, nil)
 
 	// Change Settings, then clear cache without writing.
 	node, err := ctx.Settings()
-	c.Assert(err, IsNil)
-	expect := node.Map()
+	c.Assert(err, gc.IsNil)
+	expectSettings := node.Map()
+	expectMap := convertSettings(expectSettings)
 	node.Set("change", "exciting")
 	ctx.ClearCache()
 
 	// Check that the change is not cached...
 	node, err = ctx.Settings()
-	c.Assert(err, IsNil)
-	c.Assert(node.Map(), DeepEquals, expect)
+	c.Assert(err, gc.IsNil)
+	c.Assert(node.Map(), gc.DeepEquals, expectSettings)
 
 	// ...and not written to state.
 	settings, err := s.ru.ReadSettings("u/0")
-	c.Assert(err, IsNil)
-	c.Assert(settings, DeepEquals, expect)
+	c.Assert(err, gc.IsNil)
+	c.Assert(settings, gc.DeepEquals, expectMap)
 
 	// Change again, write settings, and clear caches.
 	node.Set("change", "exciting")
 	err = ctx.WriteSettings()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	ctx.ClearCache()
 
 	// Check that the change is reflected in Settings...
-	expect["change"] = "exciting"
+	expectSettings["change"] = "exciting"
+	expectMap["change"] = expectSettings["change"]
 	node, err = ctx.Settings()
-	c.Assert(err, IsNil)
-	c.Assert(node.Map(), DeepEquals, expect)
+	c.Assert(err, gc.IsNil)
+	c.Assert(node.Map(), gc.DeepEquals, expectSettings)
 
 	// ...and was written to state.
 	settings, err = s.ru.ReadSettings("u/0")
-	c.Assert(err, IsNil)
-	c.Assert(settings, DeepEquals, expect)
+	c.Assert(err, gc.IsNil)
+	c.Assert(settings, gc.DeepEquals, expectMap)
 }
 
 type InterfaceSuite struct {
 	HookContextSuite
 }
 
-var _ = Suite(&InterfaceSuite{})
+var _ = gc.Suite(&InterfaceSuite{})
 
-func (s *InterfaceSuite) GetContext(c *C, relId int, remoteName string) jujuc.Context {
+func (s *InterfaceSuite) GetContext(c *gc.C, relId int,
+	remoteName string) jujuc.Context {
 	uuid, err := utils.NewUUID()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	return s.HookContextSuite.GetHookContext(c, uuid.String(), relId, remoteName)
 }
 
-func (s *InterfaceSuite) TestUtils(c *C) {
+func (s *InterfaceSuite) TestUtils(c *gc.C) {
 	ctx := s.GetContext(c, -1, "")
-	c.Assert(ctx.UnitName(), Equals, "u/0")
+	c.Assert(ctx.UnitName(), gc.Equals, "u/0")
 	r, found := ctx.HookRelation()
-	c.Assert(found, Equals, false)
-	c.Assert(r, IsNil)
+	c.Assert(found, gc.Equals, false)
+	c.Assert(r, gc.IsNil)
 	name, found := ctx.RemoteUnitName()
-	c.Assert(found, Equals, false)
-	c.Assert(name, Equals, "")
-	c.Assert(ctx.RelationIds(), HasLen, 2)
+	c.Assert(found, gc.Equals, false)
+	c.Assert(name, gc.Equals, "")
+	c.Assert(ctx.RelationIds(), gc.HasLen, 2)
 	r, found = ctx.Relation(0)
-	c.Assert(found, Equals, true)
-	c.Assert(r.Name(), Equals, "db")
-	c.Assert(r.FakeId(), Equals, "db:0")
+	c.Assert(found, gc.Equals, true)
+	c.Assert(r.Name(), gc.Equals, "db")
+	c.Assert(r.FakeId(), gc.Equals, "db:0")
 	r, found = ctx.Relation(123)
-	c.Assert(found, Equals, false)
-	c.Assert(r, IsNil)
+	c.Assert(found, gc.Equals, false)
+	c.Assert(r, gc.IsNil)
 
 	ctx = s.GetContext(c, 1, "")
 	r, found = ctx.HookRelation()
-	c.Assert(found, Equals, true)
-	c.Assert(r.Name(), Equals, "db")
-	c.Assert(r.FakeId(), Equals, "db:1")
+	c.Assert(found, gc.Equals, true)
+	c.Assert(r.Name(), gc.Equals, "db")
+	c.Assert(r.FakeId(), gc.Equals, "db:1")
 
 	ctx = s.GetContext(c, 1, "u/123")
 	name, found = ctx.RemoteUnitName()
-	c.Assert(found, Equals, true)
-	c.Assert(name, Equals, "u/123")
+	c.Assert(found, gc.Equals, true)
+	c.Assert(name, gc.Equals, "u/123")
 }
 
-func (s *InterfaceSuite) TestUnitCaching(c *C) {
+func (s *InterfaceSuite) TestUnitCaching(c *gc.C) {
 	ctx := s.GetContext(c, -1, "")
 	pr, ok := ctx.PrivateAddress()
-	c.Assert(ok, Equals, true)
-	c.Assert(pr, Equals, "u-0.example.com")
+	c.Assert(ok, gc.Equals, true)
+	c.Assert(pr, gc.Equals, "u-0.testing.invalid")
 	_, ok = ctx.PublicAddress()
-	c.Assert(ok, Equals, false)
+	c.Assert(ok, gc.Equals, false)
 
 	// Change remote state.
 	u, err := s.State.Unit("u/0")
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	err = u.SetPrivateAddress("")
-	c.Assert(err, IsNil)
-	err = u.SetPublicAddress("blah.example.com")
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
+	err = u.SetPublicAddress("blah.testing.invalid")
+	c.Assert(err, gc.IsNil)
 
 	// Local view is unchanged.
 	pr, ok = ctx.PrivateAddress()
-	c.Assert(ok, Equals, true)
-	c.Assert(pr, Equals, "u-0.example.com")
+	c.Assert(ok, gc.Equals, true)
+	c.Assert(pr, gc.Equals, "u-0.testing.invalid")
 	_, ok = ctx.PublicAddress()
-	c.Assert(ok, Equals, false)
+	c.Assert(ok, gc.Equals, false)
 }
 
-func (s *InterfaceSuite) TestConfigCaching(c *C) {
+func (s *InterfaceSuite) TestConfigCaching(c *gc.C) {
 	ctx := s.GetContext(c, -1, "")
-	cfg, err := ctx.Config()
-	c.Assert(err, IsNil)
-	c.Assert(cfg, DeepEquals, map[string]interface{}{"blog-title": "My Title"})
+	settings, err := ctx.ConfigSettings()
+	c.Assert(err, gc.IsNil)
+	c.Assert(settings, gc.DeepEquals, charm.Settings{"blog-title": "My Title"})
 
 	// Change remote config.
-	node, err := s.service.Config()
-	c.Assert(err, IsNil)
-	node.Set("blog-title", "Something Else")
-	_, err = node.Write()
-	c.Assert(err, IsNil)
+	err = s.service.UpdateConfigSettings(charm.Settings{
+		"blog-title": "Something Else",
+	})
+	c.Assert(err, gc.IsNil)
 
 	// Local view is not changed.
-	cfg, err = ctx.Config()
-	c.Assert(err, IsNil)
-	c.Assert(cfg, DeepEquals, map[string]interface{}{"blog-title": "My Title"})
+	settings, err = ctx.ConfigSettings()
+	c.Assert(err, gc.IsNil)
+	c.Assert(settings, gc.DeepEquals, charm.Settings{"blog-title": "My Title"})
 }
 
 type HookContextSuite struct {
@@ -611,20 +626,31 @@ type HookContextSuite struct {
 	relch    *state.Charm
 	relunits map[int]*state.RelationUnit
 	relctxs  map[int]*uniter.ContextRelation
+
+	st      *api.State
+	uniter  *apiuniter.State
+	apiUnit *apiuniter.Unit
 }
 
-func (s *HookContextSuite) SetUpTest(c *C) {
+func (s *HookContextSuite) SetUpTest(c *gc.C) {
 	s.JujuConnSuite.SetUpTest(c)
 	var err error
 	sch := s.AddTestingCharm(c, "wordpress")
 	s.service, err = s.State.AddService("u", sch)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	s.unit = s.AddUnit(c, s.service)
+
+	err = s.unit.SetPassword("password")
+	c.Assert(err, gc.IsNil)
+	s.st = s.OpenAPIAs(c, s.unit.Tag(), "password")
+	s.uniter = s.st.Uniter()
+	c.Assert(s.uniter, gc.NotNil)
+
 	// Note: The unit must always have a charm URL set, because this
 	// happens as part of the installation process (that happens
 	// before the initial install hook).
 	err = s.unit.SetCharmURL(sch.URL())
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	s.relch = s.AddTestingCharm(c, "mysql")
 	s.relunits = map[int]*state.RelationUnit{}
 	s.relctxs = map[int]*uniter.ContextRelation{}
@@ -632,34 +658,60 @@ func (s *HookContextSuite) SetUpTest(c *C) {
 	s.AddContextRelation(c, "db1")
 }
 
-func (s *HookContextSuite) AddUnit(c *C, svc *state.Service) *state.Unit {
+func (s *HookContextSuite) AddUnit(c *gc.C, svc *state.Service) *state.Unit {
 	unit, err := svc.AddUnit()
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	name := strings.Replace(unit.Name(), "/", "-", 1)
-	err = unit.SetPrivateAddress(name + ".example.com")
-	c.Assert(err, IsNil)
+	err = unit.SetPrivateAddress(name + ".testing.invalid")
+	c.Assert(err, gc.IsNil)
 	return unit
 }
 
-func (s *HookContextSuite) AddContextRelation(c *C, name string) {
+func (s *HookContextSuite) AddContextRelation(c *gc.C, name string) {
 	_, err := s.State.AddService(name, s.relch)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	eps, err := s.State.InferEndpoints([]string{"u", name})
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	rel, err := s.State.AddRelation(eps...)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	ru, err := rel.Unit(s.unit)
-	c.Assert(err, IsNil)
+	c.Assert(err, gc.IsNil)
 	s.relunits[rel.Id()] = ru
 	err = ru.EnterScope(map[string]interface{}{"relation-name": name})
-	c.Assert(err, IsNil)
-	s.relctxs[rel.Id()] = uniter.NewContextRelation(ru, nil)
+	c.Assert(err, gc.IsNil)
+	s.apiUnit, err = s.uniter.Unit(s.unit.Tag())
+	c.Assert(err, gc.IsNil)
+	apiRel, err := s.uniter.Relation(rel.Tag())
+	c.Assert(err, gc.IsNil)
+	apiRelUnit, err := apiRel.Unit(s.apiUnit)
+	c.Assert(err, gc.IsNil)
+	s.relctxs[rel.Id()] = uniter.NewContextRelation(apiRelUnit, nil)
 }
 
-func (s *HookContextSuite) GetHookContext(c *C, uuid string, relid int, remote string) *uniter.HookContext {
+func (s *HookContextSuite) GetHookContext(c *gc.C, uuid string, relid int,
+	remote string) *uniter.HookContext {
 	if relid != -1 {
 		_, found := s.relctxs[relid]
-		c.Assert(found, Equals, true)
+		c.Assert(found, gc.Equals, true)
 	}
-	return uniter.NewHookContext(s.unit, "TestCtx", uuid, relid, remote, s.relctxs)
+	context, err := uniter.NewHookContext(s.apiUnit, "TestCtx", uuid, relid, remote,
+		s.relctxs, apiAddrs)
+	c.Assert(err, gc.IsNil)
+	return context
+}
+
+func convertSettings(settings params.RelationSettings) map[string]interface{} {
+	result := make(map[string]interface{})
+	for k, v := range settings {
+		result[k] = v
+	}
+	return result
+}
+
+func convertMap(settingsMap map[string]interface{}) params.RelationSettings {
+	result := make(params.RelationSettings)
+	for k, v := range settingsMap {
+		result[k] = v.(string)
+	}
+	return result
 }
