@@ -1,19 +1,24 @@
+// Copyright 2012, 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package hook_test
 
 import (
-	. "launchpad.net/gocheck"
+	"testing"
+
+	gc "launchpad.net/gocheck"
+
 	"launchpad.net/juju-core/charm/hooks"
 	"launchpad.net/juju-core/worker/uniter/hook"
-	"testing"
 )
 
 func Test(t *testing.T) {
-	TestingT(t)
+	gc.TestingT(t)
 }
 
 type InfoSuite struct{}
 
-var _ = Suite(&InfoSuite{})
+var _ = gc.Suite(&InfoSuite{})
 
 var validateTests = []struct {
 	info hook.Info
@@ -43,14 +48,14 @@ var validateTests = []struct {
 	{hook.Info{Kind: hooks.RelationBroken}, ""},
 }
 
-func (s *InfoSuite) TestValidate(c *C) {
+func (s *InfoSuite) TestValidate(c *gc.C) {
 	for i, t := range validateTests {
 		c.Logf("test %d", i)
 		err := t.info.Validate()
 		if t.err == "" {
-			c.Assert(err, IsNil)
+			c.Assert(err, gc.IsNil)
 		} else {
-			c.Assert(err, ErrorMatches, t.err)
+			c.Assert(err, gc.ErrorMatches, t.err)
 		}
 	}
 }
