@@ -1,6 +1,11 @@
+// Copyright 2012, 2013 Canonical Ltd.
+// Licensed under the AGPLv3, see LICENCE file for details.
+
 package jujuc
 
 import (
+	"fmt"
+
 	"launchpad.net/gnuflag"
 	"launchpad.net/juju-core/cmd"
 )
@@ -10,6 +15,7 @@ type ConfigGetCommand struct {
 	cmd.CommandBase
 	ctx Context
 	Key string // The key to show. If empty, show all.
+	All bool
 	out cmd.Output
 }
 
@@ -18,16 +24,23 @@ func NewConfigGetCommand(ctx Context) cmd.Command {
 }
 
 func (c *ConfigGetCommand) Info() *cmd.Info {
+	doc := `
+When no <key> is supplied, all keys with values or defaults are printed. If
+--all is set, all known keys are printed; those without defaults or values are
+reported as null. <key> and --all are mutually exclusive.
+`
 	return &cmd.Info{
 		Name:    "config-get",
 		Args:    "[<key>]",
 		Purpose: "print service configuration",
-		Doc:     "If a key is given, only the value for that key will be printed.",
+		Doc:     doc,
 	}
 }
 
 func (c *ConfigGetCommand) SetFlags(f *gnuflag.FlagSet) {
 	c.out.AddFlags(f, "smart", cmd.DefaultFormatters)
+	f.BoolVar(&c.All, "a", false, "print all keys")
+	f.BoolVar(&c.All, "all", false, "")
 }
 
 func (c *ConfigGetCommand) Init(args []string) error {
@@ -35,19 +48,30 @@ func (c *ConfigGetCommand) Init(args []string) error {
 		return nil
 	}
 	c.Key = args[0]
+	if c.Key != "" && c.All {
+		return fmt.Errorf("cannot use argument --all together with key %q", c.Key)
+	}
+
 	return cmd.CheckEmpty(args[1:])
 }
 
 func (c *ConfigGetCommand) Run(ctx *cmd.Context) error {
-	cfg, err := c.ctx.Config()
+	settings, err := c.ctx.ConfigSettings()
 	if err != nil {
 		return err
 	}
 	var value interface{}
 	if c.Key == "" {
-		value = cfg
+		if !c.All {
+			for k, v := range settings {
+				if v == nil {
+					delete(settings, k)
+				}
+			}
+		}
+		value = settings
 	} else {
-		value, _ = cfg[c.Key]
+		value, _ = settings[c.Key]
 	}
 	return c.out.Write(ctx, value)
 }
